#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/timeb.h>

#include "fits2lc.h"


/* **************************************************************** */

float detecmap(float x_source, float y_source, char srcfun[], int last, int jemx)	
 
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                                               */        
/*      detecmap.c generates a list of detector pixels which are                 */        
/*      illuminated from a particular sky position.                              */
/*                                                                               */
/*      Niels Lund, February 2003                                                */        
/*      Updated for NJW-mask and detector support struct. Sep. 2003              */
/*                                                                               */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */

/* Illumination JEM-X detector surface for source at position (x_source, y_source). 
   Derived from skymap version 3.0 
	
*	Niels Lund, January 2003	Version 1.0
*	Danish Space Research Institute
*	
*	NOTE: The jemx variable is 0 for JEM-X1 and 1 for JEM-X2 in these routines !!
*
*	The XY-coordinate system used in this routine is aligned with 
*	the detector collimator lamellae. The microstrip pattern is 
*	approximately aligned with the lamellae. The cathode positions 
*	are (nearly) along the X-axis, the backside positions nearly 
*	along the Y-axis. The center of the detector is located at
*	position (128, 128). The central collimator crossing is
*	also at this position. The  center of the collimator falls at 
*       the corner between 4 pixel boundaries.
*
*	The unit used for the X- and Y-positions on the detector are 
*	1 mm. Actually, the X- and Y-integer positions are only 
*	approximately separated by 1 mm. A detailed map, specifying 
*   	the float values of the X- and Y-position of the four corners 
*	of each pixel have been prepared by Carl Budtz-J and are
*	applied to the data using subroutine "pixelfold". 
*
*	The full field of view of the JEM-X detector is defined by the
*	dimensions of the mask (535 mm diameter) and detector (250 mm
*	diameter) and by the mask-detector separation (about 3400 mm).
*
*       The full field of view, which has a radius of 'FOV_radius' pixels. 
*
*/

{
   int i, j, k, m, allownum, n, ring, nn, dif, sky2, iq, iv, ky, mx, mmm, iqp, ivp;
   int x20, y20, xy20, looplim, xcdif, ycdif, xyacc, p_loss, allowpix=0, njw1, njw2;
   int njwhole, strblock, njwOK, p_count=0;

   unsigned det_index, dd_index, sky_index, ss_index, r_sky2;
   unsigned mx2,  xd7, yd7, xc7, yc7, cur_index, aind;
   unsigned st_byt_in;

   static double collp30;	/* 30 times the collimator pitch */
   static double coll_h;       /* height of collimator */
   static double coll_w;       /* halfthickness of lamellae */
   static double hexa_dim;     /* flat-to-flat dimension of the mask elements */

   static double alfa, cosal, sinal, beta, cosbe, sinbe, gamma, cosga, singa;
   static double struc_h1, struc_h2;
   static double rsin60, cot60, coll_pitch, c_tiltx, c_tilty;
   
   double ic, jc, coll_r2, xdet, ydet, xmask, ymask;
   double allow, r_sky, c_fact, c_factb, m_fact, mask_r2, rm2;
   double ydet2, qmask, vmask, dif7, xcoll2;
   double xdmod7, ydmod7, xtmod7, ytmod7, c0_fact, sum, xtemp;

   double ycolbot, ycoltop, xcolbot, xcoltop, xrmask, yrmask, xmask2;
   double xrcbot, xrctop, yrcbot, yrctop, xrdet, yrdet;
   float norm, indat[2];
   
   static int first = 1;
   static int njwmask[200][200]; 
   char mhline[410], *charpt, pchar, maskfil[200], alfil[200];
   FILE *mhfil;
   
   /* initializations */

   if (first) {
      sprintf(maskfil, "%s/reference/maskhole.txt", sweethome);
/*      printf("Opening: %s", maskfil);
*/
      mhfil = fopen(maskfil, "rt");
      if (mhfil == NULL) {printf("Could not open maskhole.txt\n"); exit(0);}
      
      for (i=0; i<200; i++) {	/* read NJW definition of open mask holes */
         fgets(mhline, 410, mhfil);
         for (j=0; j<200; j+=20) {
            charpt = &mhline[j*2];
            sscanf(charpt, "%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d",
               &njwmask[i][j+0], &njwmask[i][j+1], &njwmask[i][j+2], &njwmask[i][j+3], 
               &njwmask[i][j+4], &njwmask[i][j+5], &njwmask[i][j+6], &njwmask[i][j+7], 
               &njwmask[i][j+8], &njwmask[i][j+9], &njwmask[i][j+10], &njwmask[i][j+11], 
               &njwmask[i][j+12], &njwmask[i][j+13], &njwmask[i][j+14], &njwmask[i][j+15], 
               &njwmask[i][j+16], &njwmask[i][j+17], &njwmask[i][j+18], &njwmask[i][j+19]);
         }
      }
      fclose(mhfil);
      first = 0;

      FOV_radius = -1;
      sprintf(alfil, "%s/reference/alignment.dat", sweethome);
      mhfil = fopen(alfil, "rt");
      if (mhfil == NULL) {
         printf("No alignment file found! (%s)\n", alfil);
         exit(0);
      } else {
         for (i=0; i<200; i++) {
            fgets(mhline, 200, mhfil);
            if (feof(mhfil)) goto q_end;
            sscanf(mhline, "%d", &n);
            if (n == 1) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); coll_pitch = indat[jemx];}
            if (n == 2) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); coll_h = indat[jemx];}
            if (n == 3) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); coll_w = indat[jemx];}
            if (n == 4) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); coll_r = indat[jemx];}
            if (n == 5) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); mask_r = indat[jemx];}
            if (n == 6) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); mask_height = indat[jemx];}
            if (n == 7) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); hexa_dim = indat[jemx];}
            if (n == 8) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); struc_h2 = indat[jemx];}
            if (n == 9) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); struc_h1 = indat[jemx];}
            if (n == 10) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); alfa = indat[jemx];}
            if (n == 11) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); beta = indat[jemx];}
            if (n == 12) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); gamma = indat[jemx];}
            if (n == 13) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); xshi = indat[jemx];}
            if (n == 14) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); yshi = indat[jemx];}
            if (n == 15) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); c_tiltx = indat[jemx];}
            if (n == 16) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); c_tilty = indat[jemx];}
            if (n == 21) {sscanf(mhline, "%d %f %f", &n, &indat[0], &indat[1]); FOV_radius = indat[jemx];}
         }
      }
q_end:
      if (FOV_radius < 100) {printf("alignment data not updated to contain FOV_radius, exiting!\n"); exit(0);}

      alfa *= gtorad;
      beta *= gtorad;
      gamma *= gtorad;
      
      printf("\nRead parameters: %lf %lf %lf %lf %lf %lf %lf %lf\n", 
         coll_pitch, coll_h, coll_w, coll_r, mask_r, hexa_dim, struc_h2, struc_h1);
      printf("Read parameters: %lf %lf %lf %lf %lf %lf\n", 
         alfa, beta, gamma, xshi, yshi, mask_height);
      fclose(mhfil);

   }
      
/*   coll_pitch = 7.05;
   coll_h = 57.0;
   coll_w = 0.45 * 0.5;
   coll_r =  125.0;
   mask_r = 535.0 * 0.5;
   hexa_dim = 3.3;
   struc_h2 = 21.25    height of high mask support structural members (incl. 0.25 mm mask half thickn. 
   struc_h1 = 10.25  height of low mask support structural members (incl. 0.25 mm mask half thickn. 
   alfa 	 angle between mask X-axis and Common-X 
   beta 	 angle between collimator X-axis and Common-X 
   gamma	 angle between microstrip X-axis and Common-X 
 */

   collp30 = coll_pitch * 30.0;  
   coll_r2 = coll_r * coll_r;
   mask_r2 = mask_r * mask_r;

   r_sky = mask_r + coll_r;	/* field of view radius */

   /* Common-X is the direction from JEM-X2 towards JEM-X1 */
   
   cosal = cos(alfa);
   sinal = sin(alfa);
   cosbe = cos(beta);
   sinbe = sin(beta);
   cosga = cos(gamma);
   singa = sin(gamma);
   
   c0_fact = (coll_pitch - 2.0 * coll_w) / FOV_radius; 	/* stepsize in (mm) in skymap at collimator top level */ 
      							/* (0.0519685 mm for 'FOV_radius' = 127) */
                 /* this routine calculates illumination at the detector window level (indepc = 0.0) */
   m_fact = c0_fact * mask_height  / coll_h;		/* corresponding stepsize at mask level */	
   c_factb = 0.0;	/* corresponding stepsize at bottom of collimator 
      				(this calculation is valid for the level of the detector window) */
   c_fact = c0_fact;					/* corresponding stepsize at top of collimator */
   
/*   printf("detecmap: c_fact: %6.2lf, m_fact: %6.2lf, c_factb: %6.2lf, x/y_source: %6.1f/%6.1f\n", 
      c_fact, m_fact, c_factb, x_source, y_source);
*/
   cot60 = 1.0 / (hexa_dim * 1.73205081);
   rsin60 = 2.0 * cot60;

   det_index = 0;
   sky_index = 0;
   cur_index = 0;
   allownum = 0;

    /* ***************  main loop ********************* */

   x_source -= (float)(FOV_radius);
   y_source -= (float)(FOV_radius);
         
   for (i=0; i<256; i++) {		/* loop over detector Y */

      yrdet = (double)(i) - 127.5 + yshi;
      ydet2 = yrdet * yrdet;
      
      for (j=0; j<256; j++) {		/* loop over detector X */
         
	 det_index = j * 256 + i;
         srcfun[det_index] = 0;

	 xrdet = (double)(j) - 127.5 + xshi;
         if ((ydet2 + xrdet*xrdet) > coll_r2) continue; 		/* detector radius check */
         
         xdet = xrdet * cosga - yrdet * singa;
         ydet = yrdet * cosga + xrdet * singa;
         
	 /* xdet and ydet are coordinates of a detector pixel in a coord. system aligned after Common-X */

         xyacc = 0;
         
         mx = x_source;
         mx2 = mx * mx;

         xcolbot = x_source * c_factb + xdet;
         xcoltop = x_source * c_fact + xdet;
         xcoll2 = xcoltop * xcoltop;
            
         xrmask = x_source * m_fact + xdet;
         xmask2 = xrmask * xrmask;
                
         ky = y_source;
                
         ycolbot = y_source * c_factb + ydet;
         ycoltop = y_source * c_fact + ydet;
            
         if ((xcoll2 + ycoltop * ycoltop) > coll_r2) continue;
         				/* Collimator top radius check */
                
         yrmask = y_source * m_fact + ydet;
            
         rm2 = yrmask * yrmask + xmask2;
                
         if (rm2 > mask_r2) continue;			/* the event falls outside the coded mask */
         
         srcfun[det_index] = 1;		/* indicate area on which the mask pattern is projected */

         xrcbot = xcolbot * cosbe + ycolbot * sinbe;	/* rotation through -beta! */
         yrcbot = ycolbot * cosbe - xcolbot * sinbe;
            
         xrctop = xcoltop * cosbe + ycoltop * sinbe;	/* rotation through -beta! */
         yrctop = ycoltop * cosbe - xcoltop * sinbe;
            
         /* determine open directions of collimator the collimator lamellae are located on positions:
            -126.9,-119.85, ... ,-21.15, -14.1, -7.05, 0, 7.05, 14.1, 21.15, ... , 119.85, 126.9
            both in the X- and Y-direction 
         */

         yd7 = (int)((yrcbot + collp30) / coll_pitch);	/* determine bottom collimator hole index
                   							   collp30 >detector radius */
         yc7 = (int)((yrctop + collp30) / coll_pitch);	/* determine top collimator hole index */
	    
         
         if (yc7 > yd7) continue;
               /* the event is lost due to wall crossing in the collimator */
                   
         if (yc7 < yd7) continue;
               /* the event is lost due to wall crossing in the collimator */
                   
         ydmod7 = yrcbot + collp30 - coll_pitch * yd7;	/* get remainder Ycoord within this hole */

         if (ydmod7 < coll_w) continue;
               /* the event is lost at the bottom edge of the collimator */
                   	   
         if (ydmod7 > (coll_pitch - coll_w)) continue;
               /* the event is lost at the bottom edge of the collimator */

         ytmod7 = yrctop + collp30 - coll_pitch * yc7;	/* get remainder Ycoord within this hole */
               
         if (ytmod7 < coll_w) continue;
               /* the event is lost at the top edge of the collimator */
					/* Y-collimator wall -thickness check */
                   	   
         if (ytmod7 > (coll_pitch - coll_w)) continue;
               /* the event is los  at the top edge of the collimator */
					/* Y-collimator wall +thickness check */         	   

         xd7 = (int)((xrcbot + collp30) / coll_pitch);
         xc7 = (int)((xrctop + collp30) / coll_pitch);
                    
         if (xc7 > xd7) continue;
               /* the event is lost due to wall crossing in the collimator */
                    
         if (xc7 < xd7) continue;
               /* the event is lost due to wall crossing in the collimator */

         xdmod7 = xrcbot + collp30 - coll_pitch * xd7;
            
         if (xdmod7 < coll_w) continue;
               /* the event is lost at the bottom edge of the collimator */

         if (xdmod7 > (coll_pitch - coll_w)) continue;
               /* the event is lost at the bottom edge of the collimator */

         xtmod7 = xrctop + collp30 - coll_pitch * xc7;
            
         if (xtmod7 < coll_w) continue;
               /* the event is lost at the top edge of the collimator */

         if (xtmod7 > (coll_pitch - coll_w)) continue;
               /* the event is lost at the top edge of the collimator */
                
         xmask = xrmask * cosal + yrmask * sinal; 
         ymask = yrmask * cosal - xrmask * sinal;
                
         qmask = xmask / hexa_dim - ymask * cot60;
         vmask = ymask * rsin60;
                
   	 iqp = qmask + 22501.5;
         ivp = vmask + 22501.5;
         iq = iqp - 22501;
         iv = ivp - 22501;

         njw1 = 99 + iq + iv;
         njw2 = 100 + iv;
         
         if ((njw1 < 0) || (njw1 > 199) || (njw1 < 0) || (njw1 > 199)) {
            if (p_count < 3) {
               printf("njw1/2 outside limits: %4d/%4d, i/j: %3d/%3d, x/y_source: %6.2f/%6.2f\n",
                  njw1, njw2, i, j, x_source, y_source);
               p_count++;
            }
            njw1 = njw2 = 0;   
         }
         
         if (njwmask[njw1][njw2] == 6) {
            njwOK++;
            strblock = struchk(xmask, ymask, xdet, ydet, last);
            if (strblock == 0) { 
               srcfun[det_index] += 1;
               allownum++;   
            }

         }
      }
   }
   if (last) {
      struchk(30.0, 30.0, 1.0, 1.0, 2);	/* dummy call to initiate statistics  print */
   }
   
   if ( allownum > 0) {
      norm = 1.0 / (float)(allownum);
      return(norm);
   } else return(-1.0);
}

