/*
Routine to read JEM-X data from fits file and extract source light curves
according to source positions provided in a parameter file.

Includes correction to the pulseheights for the average gain for this observation.
November 2003. Generate source light curves according to a source position
               provided in a parameter file of the form:
               RA(J2000) dec(J2000) light_curve_time_step(s)               
example:    288.7983 10.9456 900  (for a lightcurve for GRS1915+105 with 900 s time bins)

December 2003. Corrected deficiencies in the interpretation of fits files               

December  2003. Included evaluation of background in 4 comparison points around source.

January 2004.   Corrected error in sh_depth (kx should be replaced by ky in assignment of wally_z)
               
February 2004.  Included treatment of 'restricted imaging'                

March 2004. Updated jemx1/jemx2 handling in getRAdec
*/

#include<stdio.h>
#include<math.h>
#include<stdlib.h>
#include<string.h>

#include "fits2lc.h"

#define firstevent 32000.0
#define detdim 65536
#define srcnum 20
#define DATA_POINTS 25000 
#define src_events 1000000
#define bck_events 3000000

/*#include<dos.h>*/

double pi, twopi, gtorad;
double scaleX, scaleY, x2x, y2y, offX, offY, offQ, offR, phase2, xshi, yshi, indep; 
double j_RA, j_dec;
double coll_r;	/* collimator radius */ 
double mask_r;	/* radius of coded mask pattern */
double mask_height;
double pix2rad, rad2g = 180.0 / 3.14159;
int FOV_radius;

double parameters[20];	/* array with the specific  parameters for current skymap:
        parameters[0] = byt_index;	length of 'd_allow' list 

        parameters[7] = cond;		condition parameter for inclusion of collimator in backprojection 
	          		cond = 0: disregard collimator in backprojection 
                                cond = 1: include collimator
        parameters[8] = jemx; 		JEM-X unit number 
        parameters[9] = FOV_radius	Radius of FOV expressed in sky-pixels
        parameters[10] = indep;		interaction depth of photons in the detector 
           			Common-X is the direction from JEM-X2 towards JEM-X1 
        parameters[11] = alfa; 		angle between mask X-axis and Common-X 
        parameters[12] = beta;		angle between collimator X-axis and Common-X  		
        parameters[13] = gamma;		angle between microstrip X-axis and Common-X 

        parameters[14] = 0.0;		was 'Hsource' for ESTEC calibrations, now not used 
        parameters[15] = xshi;		X-shift of microstrip center relative to collimator center 
        parameters[16] = yshi;		Y-shift of microstrip center relative to collimator center 
        parameters[17] = maskoffs;	displacement of mask center spacecraft centerline, now not used
        parameters[18] = mask_height;	height of mask above detector window 
        parameters[19] = sky_version;	skymap version (must correspond to the 'detmap' version)
*/
/* 	!!!!!!!! NEW ph-thresholds for definition of energy bands !!!!!!!!!!!!!!!!!!!!!! 
              if ((ri_pha >= 1) && (ri_pha < 3) ee = 0;	
              if ((ri_pha >= 3) && (ri_pha < 5) ee = 1;
              if (ri_pha == 5) ee = 2;
              if (ri_pha >= 6) ee = 3;
                      ri_pha:   0      1      2      3      4      5      6      7
       corresp. 4096 channel:   0     32     64     96    128    192    320    576   4095 (on-board upper lim. 1200)
       corresp.  nl  channel:   0      8     16     24     32     48     80    144   1023 (300)
crude keV (22 keV in ch 500):   0    1.4    2.8    4.2    5.6    8.4   14.0   25.3  180.2  52.8
      absorbtion length (mm):      -      0.0    1.0    2.0    4.3   10.0   40.0   70.0
  absorbtion length (e-band):                1.0 mm       3.5 mm    10.0 mm    45.0 mm
X-ray raw count distribution:     0.05%   2.3%  16.4%  21.0%  26.3%  18.3%   8.1%   7.6%            (orbit 108)
                 energy band:           |    0000     |    1111     | 2222 |       3333        |    
     estmated position sigma:  inf          2.1 mm 1.6 mm 1.2 mm 0.8 mm .46 mm .46         0.6 mm
*/

/* double abs_len[4] = {1.0, 4.0, 15.0, 55.0}; *//* absorbtion length (mm) in the four energy bands */

double abs_len[5] = {0.1, 2.0, 4.3, 20.0, 80.0}; /* abs. length (mm) at the boundaries of the four energy bands */
						/* new values for restricted image version */
                                                
double x_rms[5] = {2.0, 1.5, 1.4, 1.3, 1.2}; 	/* position determination uncertainty at the boundaries*/
double y_rms[5] = {2.0, 1.5, 1.4, 1.3, 1.2}; 	/* new values for low energy version */

float acc_w[4] = {0.2, 0.4, 0.6, 0.5};		/* rel. weight of low boundary contribution to events */

int pha_sel[2][5] = { 5, 14, 24, 48, 180, 8, 24, 48, 80, 300};  /* pulseheight selection boundaries */
/* these boundaries corresponds roughly to 2, 4.2, 8.4, 14 and 35 keV (with reduced voltage setting) */ 

char s_pickmap[5][4][detdim], b_pickmap[6][4][detdim];
char initpat[detdim];
unsigned char sfmap[4][detdim];
int limit[4][101], m_max;
double snratio[4][101];
float soft_shdgr[4][detdim];
float soft_gr[detdim];
float accupat[detdim];
double soft_tot[30];
double aux_tot[30];
short shdgr[4][256][256];
char iname2[200], sweethome[200], sweethome2[200];

unsigned char fitsdat[10000000];

unsigned char src_rpha[src_events];
unsigned char src_ipha[src_events];
unsigned char src_grey[src_events];
unsigned char    src_x[src_events];
unsigned char    src_y[src_events];
double        src_time[src_events];
unsigned char bck_rpha[bck_events];
unsigned char bck_ipha[bck_events];
unsigned char bck_grey[bck_events];
unsigned char    bck_x[bck_events];
unsigned char    bck_y[bck_events];
double        bck_time[bck_events];

unsigned short lc_raw[4][DATA_POINTS], lc_backgr[4][DATA_POINTS];
float lc_rate[4][DATA_POINTS], lc_srate[4][DATA_POINTS], lc_brate[4][DATA_POINTS];
float lc_error[4][DATA_POINTS], lc_ratex[5][4][DATA_POINTS], lc_livetime[DATA_POINTS];
double lc_time[DATA_POINTS];
float T_elapse[DATA_POINTS];

short cnt_rat2indx[32000]; /* these are 125 ms light curve data extending over max 4000 s (one scw) */
char  cnt_ratfrac[32000];
char  cnt_ratdata[32000];
char  cnt_ratgrey[32000];

/* Extensions with 'full imaging', 'restricted imaging' and 'count rate' packets may alternate within
   the fits file, corresponding to the format changes taking place during the observations.
   
   If an extension of type 'count rate' or 'restricted image' is encountered while generating light
   curves with time bins of less than 8 s duration the these extensions will not be analyzed. But
   'full imaging' extensions belonging to this science window should be analyzed in the normal way. 
    
   It will be preferable to have available and analyze all the 'full imaging' and 'count rate' packets 
   before starting on the 'restricted image' packets. In this way it will be possible to allocate the 
   correct fraction of the events in a restricted image packet to one of several time bins covered by
   the image.
   
   If an extension of type 'count rate' or 'restricted image' is encountered during analysis a system
   of 4 parallel arrays: 'res_cnt2bin[32000]' 'cnt_ratfrac[]', 'cnt_ratdata[]' and 'cnt_ratgrey[]' will 
   be set up. Each entry corresponds to a 125 ms time channel, the total time covered is 4000 s, more 
   than the duration of even the longest science windows.


    -  'cnt_rat2indx' contains the index (lc_index) pointing to the output time bin which must receive 
       the events from this 125 ms channel. 
       Normally there will only be one such channel, but occasionally 2, when a 125 ms channel must
       be split between two output time bins. Such cases can be identified from the corresponding
       entry in the following table.

    -  'cnt_ratfrac' contains the fraction (in %) of the events belonging to this input channel which 
       should go to the output bin indicated in 'cnt_rat2indx'. The rest must go to the following bin. 

       These arrays can be generated on the basis of the lc_startT and lc_timestep values alone. 
       This assumes that the lc_startT value is known - and event data detected in later fits extensions
       with time stamps preceeding lc_startT will have to be discarded - a warning message should be 
       issued. 

   When this array has been set up we can proceed with the analysis of the count rate data, this involves
   filling the third fourth array:
   
    -  'cnt_ratdata' contains the count data extracted from the fits extension - placed in their correct
       position within the array as calculated from the time stamp of each time bin. In the cases where
       the 'cnt_ratfrac' value specifies that not all events should go to the specified output bin, then
       the remaining counts from this input channel should just be added to the subsequent channel in
       'cnt_ratdata'.
       
    -  'cnt_ratgrey' in the same way contains the grey filter values extracted from the fits extension
       Even if the counts for a particular input channel is split between two output bins no transfer
       of grey filter data should be done - this would be excessive.
        
   Finally we can distribute the events from a 'restricted image' into the correct time bins - in the
   lc_raw[ee][lc_index] and lc_backgr[ee][lc_index] arrays. From the known start and end times of 
   the image we can find the corresponding section of the  three arrays described above. We can then 
   identify which output bins should receive contributions from this image, and we can sum the number 
   of counts which should go to each output bin. The total number of counts to be distributed should 
   match (approximately) the number of counts in the image. The actual distribution of events must be 
   done using a MonteCarlo technique; we must assure that each output bin samples the entire image evenly.
   
   The livetime corresponding to each of the output bins must be correspondingly updated by adding 125 ms
   multiplied by the corresponding grey-filter factor to lc_livetime[ee][lc_index] for every contributing
   input channel. This accounting must be done on the count rate channel level rather than on the event 
   level as is done for full imaging data. 
*/

int src_enum = 0, src_bnum = 0;

int fitshead(unsigned char* fitsdat, unsigned char* end_ptr[], 
   	int tab_wdth[], int tab_dpth[], char E_name[], int extnum, int filelen, double st_T[]);
int MonteCarlo(int lc_index_start, int lcbins, int cut[]);

/* ***************************************************************************** */

main(argc,argv)
int argc ;
char *argv[] ;
{
	FILE *in, *sm_fil, *filist;
	FILE *spgnu ;
	FILE *spec ;
	FILE *datafil ;
	FILE *imagegnu ;
	FILE *cntsgnu ;
	FILE *hkgnu ;        
        FILE *sp_fil;
        FILE *cfil;
        FILE *cnt_file;
        FILE *src_pattern;
        FILE *listfil;
        FILE *lauxfil;
        FILE *outfil;
        FILE *gnufil;
        FILE *gnufil2;
        FILE *gnufil3;
        FILE *sumfil;
        FILE *fim;

        char sky_version = 'l', *item_st, *item_end, E_name[20], fimnavn[50], list_dat[50], list_aux[50];
        unsigned char *x_ptr, *tab_ptr, *nax1, *nax2, *dataptr;
        unsigned char *end_ptr[20], *listptr, *dataptr2, *e_ptr;
	char iname[100], oname[100], dummy[1024], statname[128], b_file[100], sumnavn[100]; 
	char spec_file[100], im_file[100], hk_file[100], filnavn[100], listname[100]; 
        
	unsigned short inb[16];
	unsigned short tm[220], mid, ssc, cal_amp, cal_n, tmdum[220];

	short grey = 31, cur_grey = -1, fits_sh, grey_rate[8];
        
        float pix_cnt[4], max_vals[4], srcsum[4], norm_val[5][4], difmax[5][4], ffrac, pixfrac;
        float src_xy[2], trans1, trans2, norm_vals[6][4], ri_duration, time_sum[5], lvtm_sum[5];
        float greysum;

        int it1, it2, it3, ssc_old, ssc_grey, pos_grey, ee, spix[5][4], bpix[6][4], item_len, acc_ev;
        int spect[256], pspect[256], pack_stat[256], sp[256], dsp[256], rspect[256], sp_r[9], dsp_r[9];
        int ipha, tot_ipspec[256], ip_pha, first_tour=1, tot_spec[256], ijmax;
        int calspec[4][256], src_spec_p[256], src_ipspec_p[256], bckgr_spec[256], bckgr_ipspec[256];
	int outblock[35], psum, first=1, hkstart, status, calseg, spin, last_nev, last_pos;
	int i, j, k, n, m, apid, apidold=-1, nsame = 0, ev_amp, an_stat, nev, ppha, ptime_i, acc_raw, bkg_raw;
	int offset, pos, hkold, hknum, hkwrap, pid, pidold, ii, jj, pickevent, esc, pidv;
        int pidevents, pidtime, b_open, jemx=0, tmnum=0, xfsum, pidselect, lolim, hilim, ipack;
        int cntindex = -1, start_ssc = -1, ire_time, ilimit, stop_ssc, old_ssc, nprint=0, ri_end=0;
        int a_cnt_index=0, dayshift, dayfix=0, sci_lobt1, num_tot[4], map_open, pidnext, it1_loop_start; 
        int OnTargetFlag, P_duration, oevnum, fillen, extnum, cnt_all_sum, ri_seq, ri_pckt=0, ri_evts=0;
        int rawpha, pha, x, y, dti, iindex, patnum, ipat, orbold, used, sev1=0, sev2=0, sev3=0, sev0=0;
        int lc_index, old_lc_index, filelen, tab_wdth[20], tab_dpth[20], cnt_sum[5], rat_index, rat_index_off;
        int xx, common_b[4], lc_index_start, lc_index_end, lc_rat_start, lc_rat_end, lc_j, n_line, grey_line[8];                int MC_distr[6], FRspec[2][8], ri_pha, shdg_out, list_out, list_code, xx_max=1;

        unsigned short SSC_1[5000], lobt2_1[5000], lobt1_1[5000], lobt0_1[5000], m_evts, n_evts2;
        unsigned short n_evts[5000], grey_1[5000], n_grey[5000], idum, lc_use[DATA_POINTS], lc_used;
        unsigned short SSC_3[200], lobt2_2[200], lobt1_2[200], lobt0_2[200], pos_gr[200], grey_3[200], delT_gr[200];
        int rowbg_1[5000], rowbg, ri_rowbg, ri_rowbg_start=-1, ri_evts_acc=0;

        float nsimple_as, nsimple_at, ncross_as, ncross_ab, msimple_s, msimple_b, mcross_s, mcross_b; 
        float cpha, src_cl_spec[256], src_error[256], src_cl_ipspec[256], src_iperror[256], e_value, e_factor;
        float ex_sim, sn_sim, ex_smear, sn_smear, ex_com, sn_com, backfact1, backfact2, backfact3, IP_tabel[256];
        float ph_corr = 1.0, e_rate, c_tilt_x, c_tilt_y, coll_height, tilt_corr, angle, lvt_all_sum;
        float srcfull[4], hdr_inf[4], src_RA, src_dec, expfact[4];
        float coll_area[5][4], bkg_aff[5][4], src_ill[5][4], m_simple_s[4], m_simple_b[4];
        float src_RAx, src_decx, xy_delta = 2.0, cmp_full; 
        float lc_rawx[5][4], lc_backgrx[5][4], comp_val[5][4], cmp_livetime=0.0, tmp, lvtime=0.0, ri_sum_dur=0.0; 
        float RAdelta[5] = {0.0, 1.0, -1.0, 0.0, 0.0}, decdelta[5] = {0, 0, 0.0, 1.0, -1.0};
        float rxy0, src_x0, src_y0, srcx, srcy;
        
	unsigned short anode ;
	unsigned int lobt[4] ;  /* we have to use 4 byte ints as idl has no unsigned */
        int nblok[256], nevent[256], cut[50], lcbins;
	int n1602 = 0, n1536 = 0, n1542 = 0 ;
	int n1608 = 0, nev1608 = 0 ;
        
	double packtime, packtime0, packrel, delta, dt, xcsum, raver, e_time, m0, m1, m2, aver, var;
        double cntratm, gc, hk_time, startT, fl_startT, rt_startT, ri_startT, runT, listT;
        double cyclow, cychigh, imageT0 = 0.0, imageT1 = 0.0, accumT, cyclownext, BCPtime, full_evtime;
        double UToffset, hk_tid, UT_time, hourofyear, hk_tider[20000], UT_res0[2];
        double xrdet, yrdet, ydet2, rmean[26], normx, norm0, aux_tot[30], rrdet;
        double lc_curT, lc_timestep, plot_offset, T_temp, Tot_elapse, st_T[20];
        double tmp_dif, tmp_dif2, tmp_dif3, tmp_dif4, error2, dtst;
        double lc_startT = -1.0, lc_endT;
        
	int dayofyear, science=0, radm0[26], radm1[26], radm2[26], irdet, fileclear;
        int pspsuma, pspsumb, st_lobt1, value, orbit, nn, dead_ano, n_sources, npix[5], fck;


/* restricted imaging count rate data */
	int n1656 = 0, nev1656 = 0 ;
	char cratebuf[406] ;
	unsigned short count, mid1, mid2 ;

/* restricted imaging */
	unsigned int n1624 = 0, nev1624 = 0, imseq, nimg1624=0, pha8 ;
	unsigned short xy, deltaxy, xbuf[5000], ybuf[5000], phabuf[5000] ;
        char iline[100], src_nam[100], src_chk[100];
	char phline[250], listnae[200] ;
        FILE *src_data, *src_check;

        /* ********************************************* */
        
        sprintf ( sweethome, "/nfs/r6/jemx/nlwork/midi_distribution_2/work" );
	sprintf ( sweethome2, "/nfs/r6/jemx/nlwork/midi_distribution_2/work/reference" );
        UToffset = 6960.0 + 5.66 - 4.34 + 0.38;	/* (hours since Jan 1st 2002 00:00:00) */ 
        
        pi = 3.14159;
        twopi = 2.0 * pi;
        gtorad = pi / 180.0;
        coll_height = 57.0;

        /* for 2003: 2003 Jan. 12th, UT 14:30:30 = 7563135 in BCPtime */
        
        for (i=0; i<256; i++) rspect[i] = pspect[i] = spect[i] = pack_stat[i] = 0;
        for (i=0; i<256; i++) calspec[0][i] = calspec[1][i] = calspec[2][i] = calspec[3][i] = 0;
        for (i=0; i<26; i++) radm0[i] = radm1[i] = radm2[i] = 0;
        for (xx=0; xx<xx_max; xx++) for (ee=0; ee<4; ee++) lc_rawx[xx][ee] = lc_backgr[xx][ee] = 0.0;
        for (i=0; i<8; i++) FRspec[0][i] = FRspec[1][i] = 0;
        for (k=0; k<4; k++) for (i=0; i<256; i++) for (j=0; j<256; j++) shdgr[k][j][i] = 0; 

        dead_ano = 0;
        jemx = 1;
        e_factor = 0.05;
        lc_timestep = 101.0;
        lc_startT = -1.0;
        fileclear = 1;
        c_tilt_x = c_tilt_y = 0.0;

	if (argc > 2) strcpy(iname, argv[1]) ;
        else {printf("fits-file source-name-file {tilt_x} {tilt_y}\n"); return;}
        
        strcpy(src_nam, argv[2]) ;
        strcpy(src_chk, argv[2]) ;
        strcpy(list_aux, argv[2]) ;
        strcpy(list_dat, argv[2]) ;
        
        strcat(src_nam, ".dat");
        strcat(src_chk, ".chk");
        strcat(list_aux, ".aux");
        strcat(list_dat, "_events.lst");
        
        src_data = fopen(src_nam, "rt");
        if (src_data == NULL) {
           printf("Could not open %s, exiting!\n", src_nam);
           return;
        }
        
        plot_offset = 0.0;

        shdg_out = list_out = 0;
        fgets(iline, 100, src_data);
        sscanf(iline, "%f %f %lf %lf %d %d", &src_RA, &src_dec, &lc_timestep, &plot_offset, &shdg_out, &list_out);
        printf("shadowgram_out: %d, event_list_out: %d\n", shdg_out, list_out);
        
      	src_check = fopen(src_chk, "rt");
        if (src_check == NULL) {
           fileclear = 1; 
           pidold = -10;
           for (i=0; i<4; i++) {           
             sprintf(filnavn, "lc_out_%1d.txt", i);
              outfil = fopen(filnavn, "wt"); 
              fprintf(outfil, "orb pid   T_stamp  T_elaps livetime  s_pix   raw  b_pix backg norm_v rate1 ");
              fprintf(outfil, " error  <r_x>  rate2 c_fact rate3  ratx1  ratx2  ratx3  ratx4 full%\n");

              fclose(outfil);
           }

           if (list_out) {
              listfil = fopen(list_dat, "wt");
              fclose(listfil);
              lauxfil = fopen(list_aux, "wt"); 
              fprintf(lauxfil, "-1   %s\n", argv[2]);
              fclose(lauxfil);
           }
        } else {
           used = -1;
           for (i=0; i<1000; i++) {
              fgets(phline, 250, src_check);
              if (feof(src_check)) goto chk_end;
              sscanf(phline, "%d %d %d", &used, &orbold, &pidold);
           }
chk_end:
    	   if (used < 0) fileclear = 1; else fileclear = 0;
           if (used <= 0) pidold = -5;
           fclose(src_check);
        }
        src_check = fopen(src_chk, "at");

        if (list_out) {
           lauxfil = fopen(list_aux, "rt");
           if (lauxfil == NULL) {
              list_code = 0;
           } else {
              for (i=0; i<1000; i++) {
                 fgets(phline, 250, lauxfil);
                 if (feof(lauxfil)) goto lst_end;
                 sscanf(phline, "%d", &list_code);   
              }
lst_end:      list_code++;
   	      fclose(lauxfil);
   	   }
           listfil = fopen(list_dat, "at");
           lauxfil = fopen(list_aux, "at");
        }
        
        jemx = -1;

        item_st = strstr(iname, "/");

/*        if (item_st == NULL) printf(" / not found\n");
        else printf(" / found at position: %2d\n", (int)(item_st - iname));
*/        
        if (item_st == NULL) item_end = iname; 
        else { 
moresl:	   item_end = item_st+1;
   	   item_st = strstr(item_end, "/");

/*           if (item_st == NULL) printf(" / not found\n");
           else printf(" / found at position: %2d\n", (int)(item_st - iname));
*/        
           if (item_st != NULL) goto moresl;
        }
        
        item_st = item_end+1;

        item_end = strstr(item_st, "_");
/*        if (item_end == NULL) {printf(" _ not found\n"); goto namex;}
        else printf(" _ found at position: %2d\n", (int)(item_end - iname));
*/        
        item_len = item_end - item_st;
        if (item_len > 5) goto namex;
        strncpy(iline, item_st, item_len);
        iline[item_len] = 0;
        jemx = atoi(iline) - 1;

        item_st = item_end+1;

        item_end = strstr(item_st, "_");
/*        if (item_end == NULL) {printf(" _ not found\n"); goto namex;}
        else printf(" _ found at position: %2d\n", (int)(item_end - iname));
*/        
        item_len = item_end - item_st;
        if (item_len > 5) goto namex;
        strncpy(iline, item_st, item_len);
        iline[item_len] = 0;
        orbit = atoi(iline);
        
        item_st = item_end+1;
        item_end = strstr(item_st, "_");
        item_len = item_end - item_st;
        if (item_len > 5) goto namex;
        strncpy(iline, item_st, item_len);
        iline[item_len] = 0;
        pid = atoi(iline);
        
        item_st = item_end+1;
        item_end = strstr(item_st, ".");
        item_len = item_end - item_st;
        if (item_len > 5) goto namex;
        strncpy(iline, item_st, item_len);
        iline[item_len] = 0;
        pidv = atoi(iline);
        
        if (jemx < 0) {
namex:     printf("Could not interpret fits file name: %s\n", iname);
           return;
        }
        
        printf("\n%s interpreted as: %1d, %3d, %4d, %3d\n", iname, jemx+1, orbit, pid, pidv);
        
        if (argc > 3) c_tilt_x = atof(argv[3]) ;           
        if (argc > 4) c_tilt_y = atof(argv[4]) ;           

        sprintf(filnavn, "%s/reference/skmap%1cc%1d_a", sweethome, sky_version, jemx+1); 
   
        strcat(filnavn, ".aux");
        datafil = fopen(filnavn, "rb");	/* opening backprojection auxiliary file */
        if (datafil == NULL) {
           printf("\nCould not open %s, goodbye\n", filnavn);
           return;
        }
   
        fread(parameters, 8, 20, datafil);
        fclose(datafil);
   
        if (dead_ano == 1) 
           sprintf(filnavn, "%s/reference/period5new/smooth%1d+anod.bin", sweethome, jemx+1);
        else sprintf(filnavn, "%s/reference/period5new/smooth%1d-anod.bin", sweethome, jemx+1);

        datafil = fopen(filnavn, "rb");
        if (datafil == NULL) {
           printf("\nCould not open %s, goodbye\n", filnavn);
           return;
        }

        fread(soft_shdgr, 4, 4*detdim, datafil);
        fread(soft_tot, 8, 30, datafil);
        fclose(datafil);
   
        sprintf(filnavn, "%s/reference/phtabl2.txt", sweethome);
        spec = fopen(filnavn, "rt");
        if (spec == NULL) {
           printf("Error opening file: %s ..... exiting!\n", filnavn) ;
           return ;
        }

        i = k = 0;
        for (i=0; i<256; i+=16) {
           fgets(phline, 128, spec);
           sscanf(phline, "%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d",
              &sp[i+0], &sp[i+1], &sp[i+2], &sp[i+3], &sp[i+4], &sp[i+5], &sp[i+6], &sp[i+7],    
              &sp[i+8], &sp[i+9], &sp[i+10], &sp[i+11], &sp[i+12], &sp[i+13], &sp[i+14], &sp[i+15]);
        }
        fclose(spec);
        
        dsp[0] = 1;
        dsp[1] = 2;
        for (i=2; i<256; i++) dsp[i] = sp[i] - sp[i-1];

        sp_r[0] = 0;
        for (i=1; i<8; i++) sp_r[i] = sp[32*i];
        sp_r[8] = sp[255];
        dsp_r[0] = 1;
        for (i=1; i<=8; i++) dsp_r[i] = sp_r[i] - sp_r[i-1];

        printf("\n ********************  Initialized pha translation table\n");
        
        sprintf(filnavn, "%s/reference/IP_tabel.txt", sweethome);
        spec = fopen(filnavn, "rt");
        if (spec == NULL) {
           printf("Error opening file: %s ..... exiting!\n", filnavn) ;
           return ;
        }

        i = k = 0;
        for (i=0; i<256; i++) {
           fgets(phline, 128, spec);
           sscanf(phline, "%d %d %d %f %f", &j, &k, &n, &cpha, &IP_tabel[i]);
        }
        fclose(spec);

        aux_tot[17] = pha_sel[jemx][0];	/* pulseheight selection boundaries */
        aux_tot[18] = pha_sel[jemx][1];
        aux_tot[19] = pha_sel[jemx][2];
        aux_tot[20] = pha_sel[jemx][3];
        aux_tot[21] = pha_sel[jemx][4];
        aux_tot[22] = jemx;
   
           /* verify availability of input fits file */
           
        in = fopen(iname,"rb") ;
        if ( in == NULL) {
           printf("Error opening file: %s ..... exiting!\n", iname) ;
           return ;
        }
           /* generate illumination maps corresponding to specified source and 4 neighbor points */

        for (xx=0; xx<xx_max; xx++) {
           if (xx == 0) {
              src_RAx = src_RA;
              src_decx = src_dec;
           } else {
              src_RAx = RAdelta[xx];
              src_decx = decdelta[xx];
           }
            
/*           printf("Calling src_map for xx = %1d\n", xx);
*/           
           npix[xx] = src_map(jemx, src_RAx, src_decx, orbit, pid, initpat, sfmap, norm_vals[xx], src_xy);
           
           if (xx == 0) {
              src_x0 = src_xy[0];
              src_y0 = src_xy[1];
           
              rxy0 = sqrt((double)(src_x0*src_x0 + src_x0*src_x0));
              if (rxy0 < 0.01) {
                 RAdelta[1] = xy_delta * pix2rad * rad2g;
                 RAdelta[2] = -xy_delta * pix2rad * rad2g;
                 RAdelta[3] = 0;
                 RAdelta[4] = 0;
                 decdelta[4] = xy_delta * pix2rad * rad2g;
                 decdelta[3] = -xy_delta * pix2rad * rad2g;
                 decdelta[2] = 0;
                 decdelta[1] = 0;
              } else {
                 srcx = src_x0 + xy_delta * src_x0 / rxy0;		/* away from axis */
                 srcy = src_y0 + xy_delta * src_y0 / rxy0;
                 getRAdec(srcx, srcy, jemx, &RAdelta[1], &decdelta[1]);
                 srcx = src_x0 - xy_delta * src_x0 / rxy0;		/* towards axis */
                 srcy = src_y0 - xy_delta * src_y0 / rxy0;
                 getRAdec(srcx, srcy, jemx, &RAdelta[2], &decdelta[2]);
                 srcx = src_x0 + xy_delta * src_y0 / rxy0;		/* to left */
                 srcy = src_y0 + xy_delta * src_x0 / rxy0;
                 getRAdec(srcx, srcy, jemx, &RAdelta[3], &decdelta[3]);
                 srcx = src_x0 - xy_delta * src_y0 / rxy0;		/* to right */
                 srcy = src_y0 - xy_delta * src_x0 / rxy0;
                 getRAdec(srcx, srcy, jemx, &RAdelta[4], &decdelta[4]);
              }
           }
        
/*           printf("norm_vals[%1d] at return from src_map: %10.4f, %10.4f, %10.4f, %10.4f\n",
              xx, norm_vals[xx][0], norm_vals[xx][1], norm_vals[xx][2], norm_vals[xx][3]);
*/
           if (npix[xx] < 0) {
              fprintf(src_check, "%d %4d %3d %s - not visible\n",
                 -fileclear,orbit, pid, argv[2], src_RAx, src_decx);
              fclose(src_check); 
              printf("%d Orbit: %4d, pid: %3d, Source: %s, RA/dec: %7.3f/%-7.3f - not visible\n",
                 -fileclear, orbit, pid, argv[2], src_RAx, src_decx);
              return;
           } else {
              if (xx == 0) {
                 fprintf(src_check, "1 %4d %3d %s - T_step %10.4lf s - lc_offset: %10.5lf (days)\n",
                    orbit, pid, argv[2], src_RAx, src_decx, lc_timestep, plot_offset);
                 fclose(src_check);
              }
        printf("1 Orbit: %4d, pid: %3d, Source: %s, RA/dec: %7.3f/%-7.3f, timestep %10.4lf,lc_offset: %10.5lf\n",
                 orbit, pid, argv[2], src_RAx, src_decx, lc_timestep, plot_offset);
           }
           
           pixfrac = npix[0] / 9400.0;
           expfact[0] = 1.0 + (1.0 - pixfrac) * 0.30;
           pixfrac = npix[1] / 9400.0;
           expfact[1] = 1.0 + (1.0 - pixfrac) * 0.27;
           pixfrac = npix[2] / 9400.0;
           expfact[2] = 1.0 + (1.0 - pixfrac) * 0.05;
           pixfrac = npix[3] / 9400.0;
           expfact[3] = 1.0 + (1.0 - pixfrac) * 0.10;

           /* collimator transmission in X */
           angle = atan2(src_xy[0], mask_height);
           trans1 = (6.6 - coll_height * tan(angle)) / 6.6;
           trans2 = (6.6 - coll_height * tan(angle + c_tilt_x * gtorad)) / 6.6;
           if ((trans1 > 0.0) && (trans2 > 0.0)) {
              tilt_corr = trans2 / trans1; 
           } else {
              printf("collimator correction undefined\n");
              return;
           }
        
           /* collimator transmission in Y */
           angle = atan2(src_xy[1], mask_height);
           trans1 = (6.6 - coll_height * tan(angle)) / 6.6;
           trans2 = (6.6 - coll_height * tan(angle + c_tilt_y * gtorad)) / 6.6;
           if ((trans1 > 0.0) && (trans2 > 0.0)) {
              tilt_corr *= trans2 / trans1; 
           } else {
              printf("collimator correction undefined\n");
              return;
           }

           	/* generate pixel selection maps for source and background filters */
           ijmax = genmap(initpat, sfmap, norm_vals[xx], coll_area[xx], bkg_aff[xx], src_ill[xx], 
                s_pickmap[xx], b_pickmap[xx], spix[xx], bpix[xx], difmax[xx]);

/*           printf("norm_vals[%1d] at return from genmap:  %10.4f, %10.4f, %10.4f, %10.4f\n",
              xx, norm_vals[xx][0], norm_vals[xx][1], norm_vals[xx][2], norm_vals[xx][3]);
*/


        }	      /* end of loop over 5 positions */
        
           	/* generate common background pixel selection map for 5 background filters */

        common_b[ee] = 0;
        for (ee=0; ee<4; ee++) {
           for (i=0; i<detdim; i++) {
              if ((b_pickmap[0][ee][i] == 1) && (b_pickmap[1][ee][i] == 1) &&
                  (b_pickmap[2][ee][i] == 1) && (b_pickmap[3][ee][i] == 1) &&
                  (b_pickmap[4][ee][i] == 1)) {
                     common_b[ee]++;
                     b_pickmap[5][ee][i] = 1; 
              } else b_pickmap[5][ee][i] = 0; 
                  
           }
        }
               
        sprintf(filnavn, "spectra.txt");
        gnufil = fopen(filnavn, "wt");
   
        patnum = 1;
/*        for (ipat=0; ipat<patnum; ipat++) {
*/
   	ipat = 0;	
           
        ph_corr = 1.0;

        /* here we read the entire fits-file into memory */

        filelen = fread(fitsdat, 1, 10000000, in);
        dataptr = fitsdat;
        for (i=0; i<20; i++) {
           end_ptr[i] = fitsdat + filelen;
           tab_wdth[i] = 0;
           tab_dpth[i] = 0;
        }


        status = fitshead(fitsdat, end_ptr, tab_wdth, tab_dpth, E_name, extnum, filelen, st_T);
        
        if (status == 0) {
        
           if ((tab_dpth[5] > 0) && (tab_dpth[2] > 0) && (tab_dpth[0] > 0)) {
              if ((st_T[5] <= st_T[2]) & (st_T[5] <= st_T[0])) lc_startT = st_T[5];
              if ((st_T[2] <= st_T[5]) & (st_T[2] <= st_T[0])) lc_startT = st_T[2];
              if ((st_T[0] <= st_T[5]) & (st_T[0] <= st_T[2])) lc_startT = st_T[0];
              goto init_lc;              
           }
           if ((tab_dpth[5] > 0) && (tab_dpth[2] > 0)) {
              if (st_T[5] <= st_T[2]) lc_startT = st_T[5]; else lc_startT = st_T[2];
              goto init_lc;              
           }
           if ((tab_dpth[5] > 0) && (tab_dpth[0] > 0)) {
              if (st_T[5] <= st_T[0])  lc_startT = st_T[5]; else  lc_startT = st_T[0];
              goto init_lc;              
           }
           if ((tab_dpth[2] > 0) && (tab_dpth[0] > 0)) {
              if (st_T[2] <= st_T[0])  lc_startT = st_T[2]; else  lc_startT = st_T[0];
              goto init_lc;              
           }
           if (tab_dpth[5] > 0) {
              lc_startT = st_T[5]; 
              goto init_lc;              
           }
           if (tab_dpth[2] > 0) {
              lc_startT = st_T[2]; 
              goto init_lc;              
           }
        } else {
           printf("Bad status on return from fitshead: %d\n", status);
           exit(0);
        }
        
init_lc:   ;
           /* global initializations */
        j = lc_startT / lc_timestep;		/* calculate fractional part of first time channel */
        lc_curT = lc_startT - j * lc_timestep;
        lc_index = old_lc_index = 0;
        e_rate = 25;	/* typical background count rate */
        Tot_elapse = 0.0;
        ri_pckt = 0;
           
        for (j=0; j<DATA_POINTS; j++) {
           for (ee=0; ee<4; ee++) lc_raw[ee][j] = lc_backgr[ee][j] = 0;
           lc_livetime[j] = T_elapse[j] = 0.0;
           lc_time[j] = lc_startT - lc_curT + lc_timestep * j;
        }
              
        for (j=0; j<32000; j++) {
           T_temp = 0.125 * j;	/* 0.125 s is the width of the count rate channels */
           k = T_temp / lc_timestep;
           if (lc_timestep <= 0.125) k = j;
           if (k > DATA_POINTS) k = DATA_POINTS;
           cnt_rat2indx[j] = k;
           cnt_ratfrac[j] = -100;			/* '-' to mark this channel as being not yet filled */
           if (lc_timestep <= 0.125) continue;
           T_temp -= k * lc_timestep;
           if ((lc_timestep - T_temp) < 0.125) cnt_ratfrac[j] = -(char)((lc_timestep - T_temp) / 0.00125); 
        }
           
        for (j=0; j<256; j++) tot_spec[j] = tot_ipspec[j] = 0;

        /* FFFFFFFFFFFFFFFFF START OF FULL IMAGING DECODING FFFFFFFFFFFFFFFFFFFFF */

full:   if (tab_dpth[5] == 0) goto rate;
        dataptr = end_ptr[7];	/* analyze FULL-SRW - if available */

        if ((dataptr - fitsdat) >= filelen) goto skip3;

        for (it1=0; it1<tab_dpth[7]; it1++) {
           fits_sh    = *dataptr * 256 + *(dataptr+1);
           SSC_3[it1]   = (int)(fits_sh & 0x7FFF) + (((int)(fits_sh & 0x8000) + 0x8000) & 0x8000);

           fits_sh    = *(dataptr+4) * 256 + *(dataptr+5);
           lobt2_2[it1] = (int)(fits_sh & 0x7FFF) + (((int)(fits_sh & 0x8000) + 0x8000) & 0x8000);

           fits_sh    = *(dataptr+6) * 256 + *(dataptr+7);
           lobt1_2[it1] = (int)(fits_sh & 0x7FFF) + (((int)(fits_sh & 0x8000) + 0x8000) & 0x8000);

           fits_sh    = *(dataptr+8) * 256 + *(dataptr+9);
           lobt0_2[it1] = (int)(fits_sh & 0x7FFF) + (((int)(fits_sh & 0x8000) + 0x8000) & 0x8000);              
             
           fits_sh    = *(dataptr+10) * 256 + *(dataptr+11);
           pos_gr[it1]  = (int)(fits_sh & 0x7FFF) + (((int)(fits_sh & 0x8000) + 0x8000) & 0x8000);

           grey_3[it1]  = *(dataptr+12);

           delT_gr[it1] = *(dataptr+13);

/*           if (it1 < 20) printf("ssc: %5d, lobt2/1/0: %5d %5d %5d, position: %3d, grey: %2d, deltaT: %4d\n",
              SSC_3[it1], lobt2_2[it1], lobt1_2[it1], lobt0_2[it1], pos_gr[it1], grey_3[it1], delT_gr[it1]);
*/
           dataptr += tab_wdth[7];
           if ((dataptr - fitsdat) > filelen) {
              printf("7 dataptr-fitsdat: %12d, filelen: %12d\n", (int)(dataptr-fitsdat), filelen);
              return;
           } 
/*           printf("%3d SSC3: %6d, pos_gr: %6d, grey: %2d, dT: %4d\n", 
              it1, SSC_3[it1], pos_gr[it1], grey_3[it1], delT_gr[it1]);
*/
        }
        
skip3:
   	dataptr = end_ptr[5];	/* analyze FULL-PRW - if available */
        
        if ((dataptr - fitsdat) >= filelen) goto rate;

        acc_ev = acc_raw = bkg_raw = 0;
        for (it1=0; it1<tab_dpth[5]; it1++) {
           fits_sh    = *dataptr * 256 + *(dataptr+1);
           SSC_1[it1]   = (int)(fits_sh & 0x7FFF) + (((int)(fits_sh & 0x8000) + 0x8000) & 0x8000);

           fits_sh    = *(dataptr+4) * 256 + *(dataptr+5);
           lobt2_1[it1] = (int)(fits_sh & 0x7FFF) + (((int)(fits_sh & 0x8000) + 0x8000) & 0x8000);

           fits_sh    = *(dataptr+6) * 256 + *(dataptr+7);
           lobt1_1[it1] = (int)(fits_sh & 0x7FFF) + (((int)(fits_sh & 0x8000) + 0x8000) & 0x8000);

           fits_sh    = *(dataptr+8) * 256 + *(dataptr+9);
           lobt0_1[it1] = (int)(fits_sh & 0x7FFF) + (((int)(fits_sh & 0x8000) + 0x8000) & 0x8000);              

/*           if ((it1 < 10) || ((it1%100) == 0)) printf("%5d SSC: %6d,  ", it1, SSC_1[it1]);
*/
           fits_sh    = *(dataptr+10) * 256 + *(dataptr+11);
           rowbg_1[it1] = ((int)(fits_sh & 0x7FFF) + (((int)(fits_sh & 0x8000) + 0x8000) & 0x8000)) << 16;
           rowbg_1[it1] += *(dataptr+12) * 256 + *(dataptr+13);
/*           if ((it1 < 10) || ((it1%100) == 0)) printf("rowbg: %8d,  ", rowbg_1[it1]);
*/
           fits_sh    = *(dataptr+14) * 256 + *(dataptr+15);
           n_evts[it1]  = (int)(fits_sh & 0x7FFF) + (((int)(fits_sh & 0x8000) + 0x8000) & 0x8000);
/*           if ((it1 < 10) || ((it1%100) == 0)) printf("n_evts: %2d,  ", n_evts[it1]);
*/
           fits_sh    = *(dataptr+18) * 256 + *(dataptr+19);
           n_grey[it1]  = (int)(fits_sh & 0x7FFF) + (((int)(fits_sh & 0x8000) + 0x8000) & 0x8000);
/*           if ((it1 < 10) || ((it1%100) == 0)) printf("ngrey: %2d,  ", n_grey[it1]);
*/         
           grey_1[it1]  = *(dataptr+20);
           
/*           if ((it1 < 10) || ((it1%100) == 0)) printf("grey: %2d\n", grey_1[it1]);
*/
/*           if (it1 < 20) printf("ssc: %5d, lobt2/1/0: %5d %5d %5d, row_begin: %6d, events: %3d, grey: %2d\n",
              SSC_1[it1], lobt2_1[it1], lobt1_1[it1], lobt0_1[it1], rowbg_1[it1], n_evts[it1], grey_1[it1]);
*/
           dataptr += tab_wdth[5];

           if ((dataptr - fitsdat) > filelen) {
              printf("dataptr-fitsdat: %12d, filelen2: %12d\n", (int)(dataptr-fitsdat), filelen);
              return;
           } 
        }

        it3 = 0;
        ssc_old = -10;
        dataptr2 = end_ptr[6];	/* analyze FULL-RAW - if available */           
        if ((dataptr2 - fitsdat) >= filelen) goto rate;

        ssc_grey = 999999999;
/*        printf("Starting loop over %4d packets with full events\n", tab_dpth[5]);
*/
        for (it1=0; it1<tab_dpth[5]; it1++) {
/*           dataptr2 = fitsdat + rowbg_1[it1] * tab_wdth[6];
*/
           if ((dataptr2 - fitsdat) > filelen) {
              printf("full_e loop: dataptr2-fitsdat: %12d, filelen6: %12d\n", (int)(dataptr2-fitsdat), filelen);
              return;
           } 
           grey = grey_1[it1];
           
           ssc = SSC_1[it1];
           if (ssc_grey < ssc) it3++;
              
           if (it3 < tab_dpth[7]) ssc_grey = SSC_3[it3]; else ssc_grey = 999999999;
           

           fl_startT = (double)(lobt2_1[it1])*65536.0 + (double)(lobt1_1[it1]) + (double)(lobt0_1[it1])/65536.0;
/*           printf("%d fl_startT: %14.6lf\n", it1, fl_startT);
*/           
           runT = listT = fl_startT;
           
/*           if ((it1%100) == 0) printf("Starting loop over %4d full events\n", n_evts[it1]);
*/           
           for (it2=0; it2<n_evts[it1]; it2++) {
           
              if (ssc_grey == ssc) {
                 if (it2 >= pos_gr[it3]) grey = grey_3[it3];
              }
              
              x = *(dataptr2);
              y = *(dataptr2+1);
              rawpha = *(dataptr2+2);
              ri_pha = (int)(rawpha/32);
              FRspec[0][ri_pha]++;

              fits_sh = *(dataptr2+3) * 256 + *(dataptr2+4);
              dti = (int)(fits_sh & 0x7FFF) + (((int)(fits_sh & 0x8000) + 0x8000) & 0x8000);

              dataptr2 += tab_wdth[6];
                 
              if (rawpha < 2) ppha = 0;
              else ppha = sp[rawpha] - (jj % dsp[rawpha]);
              
              cpha = pha_xy(x, y, ppha, jemx, ipat, 0) * ph_corr;

              e_value = cpha * e_factor;
              for (ip_pha=0; ip_pha<256; ip_pha++) if (e_value < IP_tabel[ip_pha]) goto loopout1;
loopout1:     ;

              iindex = x * 256 + y;

              pha = (int)(cpha / 4.0);
              if (pha > 254) pha = 255;
                            
/*              if ((acc_ev%1000) == 0) 
                 printf("%6d x/y: %3d/%3d, rawpha: %4d, dti: %6d, pha: %3d, acc_ev: %8d, lc_acc/bkg: %8d / %8d\n",
                 	acc_ev, x, y, rawpha, dti, pha, acc_ev, acc_raw, bkg_raw);
*/  
              pickevent = 0;
                             
              lc_curT += (double)(dti) / 8192.0;
              runT += (double)(dti) / 8192.0;
              listT += (double)(dti) / 8192.0;
              Tot_elapse += (double)(dti) / 8192.0;
              lc_index = lc_curT / lc_timestep;

              T_elapse[lc_index] += (float)(dti) / 8192.0;
              
              lc_livetime[lc_index] += (double)(dti) / 8192.0 * (grey+1) / 32.0 *
                 (1.0 - 0.109 * (0.9634 - grey * 0.00293) / 0.87 - 0.000188 * e_rate);

              cmp_livetime += (double)(dti) / 8192.0 * (grey+1) / 32.0 *
                 (1.0 - 0.109 * (0.9634 - grey * 0.00293) / 0.87 - 0.000188 * e_rate);

              iindex = 256 * (int)(x) + (int)(y);

              ee = -1;
/*              if ((pha >= pha_sel[jemx][0]) && (pha < pha_sel[jemx][1])) ee = 0;
              if ((pha >= pha_sel[jemx][1]) && (pha < pha_sel[jemx][2])) ee = 1;
              if ((pha >= pha_sel[jemx][2]) && (pha < pha_sel[jemx][3])) ee = 2;
              if ((pha >= pha_sel[jemx][3]) && (pha < pha_sel[jemx][4])) ee = 3;
*/
              if ((ri_pha >= 1) && (ri_pha < 3)) {
                 ee = 0;	
                 sev0++;
                 shdgr[0][x][y]++;
              }
              if ((ri_pha >= 3) && (ri_pha < 5)) {
                 ee = 1;
                 sev1++;
                 shdgr[1][x][y]++;
              }
              if (ri_pha == 5) {
                 ee = 2;
                 sev2++;
                 shdgr[2][x][y]++;
              }
              if (ri_pha >= 6) {
                 ee = 3;
                 sev3++;
                 shdgr[3][x][y]++;
              }

              if (ee >= 0) {
                 acc_ev++;
                 if (soft_shdgr[ee][iindex] > 0.0) {
                    tot_ipspec[ip_pha]++;
                    tot_spec[(int)(cpha / 4.0)]++;
                 }

                 if (s_pickmap[0][ee][iindex] == 1) {
                    acc_raw++;
                    lc_raw[ee][lc_index]++; 
                 }

                 if (b_pickmap[0][ee][iindex] == 1) {
                    bkg_raw++;
                    lc_backgr[ee][lc_index]++;
                 }

                 if (list_out) {
                    if (initpat[iindex] > 0) 
                       fprintf(listfil, "%3d %14.4lf %3d %3d %4.1f\n", 
                          sfmap[iindex]+1, listT, rawpha, list_code, (float)(grey));
                    else
                       fprintf(listfil, "%3d %14.4lf %3d %3d %4.1f\n", 
                          0, listT, rawpha, list_code, (float)(grey));
                 }
                 
                 for (xx=1; xx<xx_max; xx++) {
                    if (s_pickmap[xx][ee][iindex] == 1) {
                       lc_rawx[xx][ee] += 1.0; 
                    }
                    if (b_pickmap[xx][ee][iindex] == 1) {
                       lc_backgrx[xx][ee] += 1.0;
                    }
                 }
              }

            
              ipha = (int)(cpha/4.0 + 0.5);
                 
              if (s_pickmap[0][ee][iindex] == 1) {
                 src_rpha[src_enum] = rawpha;
                 src_ipha[src_enum] = ip_pha;
                 src_x[src_enum]    = x;
                 src_y[src_enum]    = y;
                 src_time[src_enum] = runT;
                 src_spec_p[ipha]++;
                 src_ipspec_p[ip_pha]++;
                 m_simple_s[ee]++; 
                 if (src_enum < src_events-1) src_enum++;
              }
              if (b_pickmap[0][ee][iindex] == 1) {
                 bck_rpha[src_bnum] = rawpha;
                 bck_ipha[src_bnum] = ip_pha;
                 bck_x[src_bnum]    = x;
                 bck_y[src_bnum]    = y;
                 bck_time[src_bnum] = runT;
                 bckgr_spec[ipha]++;
                 bckgr_ipspec[ip_pha]++;
                 m_simple_b[ee]++;
                 if (src_bnum < bck_events-1) src_bnum++;
              }
              
           }
           if ((runT - fl_startT) > 0.0) e_rate = n_evts[it1] / (runT - fl_startT);
           else e_rate = 25.0;
           if (lc_index > old_lc_index) {
/*              printf("%3d  Event rate: %6.1f (cts/s),   livetime: %6.1f (s)\n", 
                 old_lc_index, e_rate, lc_livetime[old_lc_index]);
*/
              old_lc_index = lc_index;
           }
        }
        cmp_full = cmp_livetime;
        
/*        printf("Completed extraction of 'full' events into lightcurves\n");
*/
        goto rate;

        /* ----------------- END OF FULL IMAGING DECODING --------------------------- */

restricted:
   	/* RRRRRRRRRRRRRRRRR START OF RESTRICTED IMAGING DECODING RRRRRRRRRRRRRRRRRRR */
                
        it1_loop_start = ri_end = 0;

next_image:
   	ri_evts = 0;
        for (i=0; i<6; i++) MC_distr[i] = 0;
        
        for (it1=it1_loop_start; it1<tab_dpth[2]; it1++) {
           dataptr = end_ptr[2] + it1 * tab_wdth[2];	/* analyze REST-PRW - if available */           
           if ((dataptr - fitsdat) >= filelen) {
              it1_loop_start = tab_dpth[2];
              ri_end++;
              goto ri_extract;
           }
           
           fits_sh    = *dataptr * 256 + *(dataptr+1);
           SSC_1[it1]   = (int)(fits_sh & 0x7FFF) + (((int)(fits_sh & 0x8000) + 0x8000) & 0x8000);

           fits_sh    = *(dataptr+4) * 256 + *(dataptr+5);
           lobt2_1[it1] = (int)(fits_sh & 0x7FFF) + (((int)(fits_sh & 0x8000) + 0x8000) & 0x8000);

           fits_sh    = *(dataptr+6) * 256 + *(dataptr+7);
           lobt1_1[it1] = (int)(fits_sh & 0x7FFF) + (((int)(fits_sh & 0x8000) + 0x8000) & 0x8000);

           fits_sh    = *(dataptr+8) * 256 + *(dataptr+9);
           lobt0_1[it1] = (int)(fits_sh & 0x7FFF) + (((int)(fits_sh & 0x8000) + 0x8000) & 0x8000);              
              
           fits_sh    = *(dataptr+10) * 256 + *(dataptr+11);
           ri_rowbg = ((int)(fits_sh & 0x7FFF) + (((int)(fits_sh & 0x8000) + 0x8000) & 0x8000)) << 16;
           fits_sh    = *(dataptr+12) * 256 + *(dataptr+13);
           ri_rowbg += ((int)(fits_sh & 0x7FFF) + (((int)(fits_sh & 0x8000) + 0x8000) & 0x8000));
           /* ?????????????? Check this - are there problems decoding 32 bit ints ????????????????????????? */

           if (ri_rowbg_start < 0) ri_rowbg_start = ri_rowbg;

           fits_sh    = *(dataptr+14) * 256 + *(dataptr+15);
           m_evts  = (int)(fits_sh & 0x7FFF) + (((int)(fits_sh & 0x8000) + 0x8000) & 0x8000);

           ri_seq    = *(dataptr+20) * 256 + *(dataptr+21);

           if ((ri_seq == 0x8000) && (it1 > it1_loop_start)) {
              ri_pckt++;	/* count ri_images */
/*              printf("RI_image: %3d, evts: %4d, startT: %10.5lf, 1_rt_indx: %5d, 1_lc_indx: %5d\n",
                 ri_pckt-1, ri_evts, ri_startT-lc_startT, lc_rat_start, lc_index_start);
*/
              it1_loop_start = it1;
              goto ri_extract;
           }
           
           ri_evts += m_evts;

/*           printf("ri#: %3d, events: %4d, it1: %4d, it1_loop_start: %4d, rowbg: %8d, \n", 
              ri_pckt, ri_evts, it1, it1_loop_start, ri_rowbg - ri_rowbg_start);
*/
           ri_startT = (double)(lobt2_1[it1])*65536.0 + (double)(lobt1_1[it1]) + (double)(lobt0_1[it1])/65536.0;
           listT = ri_startT;
           lc_rat_start = (ri_startT - lc_startT) / 0.125;           
           lc_index_start = cnt_rat2indx[lc_rat_start];
        }
        it1_loop_start = tab_dpth[2];
        ri_end++;

           /* ---------------- START OF DECODING OF ONE COMPLETE RESTRICTED IMAGE  ---------------------- */

ri_extract:
/*        printf("%3d, time: %10.5lf, ls_rat_start: %5d, lc_index_start: %5d, ri_end: %1d\n",
           ri_pckt, ri_startT-lc_startT, lc_rat_start, lc_index_start, ri_end);
*/
   	if (ri_end > 1) goto end;
           
        n_evts2 = 0;
        for (j=lc_rat_start; j<32000; j++) { /* loop over count rate channels corresp to this image */
           if (cnt_ratfrac[j] < 0) {
              printf("attempting to access uninitialized count rate channel: %5d\n", j);
              printf("collected %5d events, required to collect %5d events\n", n_evts2, ri_evts);
              for (k=-10; k<10; k++) printf("%+4d ", cnt_ratfrac[j+k]);
              printf("\n");
              if (n_evts2 < (int)((float)(ri_evts) * 0.95)) {
                 ri_evts = 0;
                 ri_sum_dur = 0.0;
                 lc_rat_end = j-1;
                 goto next_image;
              } else goto loopout2;
           }
           n_evts2 += cnt_ratdata[j];	/* count events until match with content of restricted image */
           greysum += cnt_ratgrey[j];	/* form sum of grey filter indicators */
           lc_rat_end = j;
           if (n_evts2 >= ri_evts) goto loopout2;
        }
loopout2:  ;
        if ((lc_rat_end == 31999) && (n_evts2 < ri_evts)) {
           printf("RI_image %3d. Missing count rate data: RI_events: %4d, rate_events: %4d\n", 
              ri_pckt, ri_evts, n_evts2);

           goto end;
        }
        
        greysum /= (float)(lc_rat_end - lc_rat_start);
           
        lc_index_end = cnt_rat2indx[lc_rat_end];
/*        printf("lc_rat_end: %6d, lc_index_end: %3d\n", lc_rat_end, lc_index_end);
*/        
        lcbins = lc_index_end - lc_index_start;
        if (lcbins > 4) {
           printf("Restricted image %3d discarded, covered %d lc-bins\n", ri_pckt, lcbins);
           ri_evts = 0;
           ri_sum_dur = 0.0;
           goto next_image;
        }
           
        k = lc_rat_end - lc_rat_start;
        ri_duration = (float)(k) * 0.125; 
        ri_sum_dur += ri_duration;
           
/*        if ((it1%10) == 0)
           printf("125 ms count rate ch. corresp. to res.image: %4d, time: %10.4lf d, duration: %6.3f s\n", 
              k, ri_startT, ri_duration);
*/              
        if ((cnt_ratfrac[lc_rat_end] < 100) && (cnt_ratfrac[lc_rat_end] > 0)) {
           lc_index_end++;
           lcbins++;
        }
            
        for (j=0; j<=lcbins; j++) {
           cnt_sum[j] = 0;
           time_sum[j] = 0.0;
           lvtm_sum[j] = 0.0;
        } 
           
           /* calculate distribution of the image counts among the affected light curve channels */

        for (j=lc_rat_start;  j<=lc_rat_end; j++) {
           if (cnt_ratfrac[j] < 0) {
              printf("Decoding attempts to use uninitialized count rate bins\n");
              return;
           }
              
           lc_j = cnt_rat2indx[j] - lc_index_start;
           e_rate += cnt_ratdata[j] - e_rate / 8.0;	/* make rolling mean over 8 */
           lvtime = 0.125 * (cnt_ratgrey[j]+1) / 32.0 *
              (1.0 - 0.109 * (0.9634 - cnt_ratgrey[j] * 0.00293) / 0.87 - 0.000188 * e_rate);

           if (cnt_ratfrac[j] == 100) {
              ffrac = (float)(cnt_ratfrac[j]) / 100.0;
              cnt_sum[lc_j] += cnt_ratdata[j];
              time_sum[lc_j] += 0.125; 
              lvtm_sum[lc_j] += lvtime;
           } else {
              ffrac = (float)(cnt_ratfrac[j]) / 100.0;
              cnt_sum[lc_j] += cnt_ratdata[j] * ffrac;
              time_sum[lc_j] += 0.125 * ffrac; 
              lvtm_sum[lc_j] += lvtime * ffrac;
              cnt_sum[lc_j+1] += cnt_ratdata[j] * (1.0 - ffrac);
              time_sum[lc_j+1] += 0.125 * (1.0 - ffrac); 
              lvtm_sum[lc_j+1] += lvtime * (1.0 - ffrac);
           }
/*              if (((it1%10) == 0) && ((j%25) == 0))
                 printf("lvtime: %10.3f, ffrac: %5.2f, j: %5d, lc_j: %5d\n", lvtime, ffrac, j, lc_j);
*/
   	}

        cnt_all_sum = 0;
        lvt_all_sum = 0;
        for (j=0; j<=lcbins; j++) {
           cnt_all_sum += cnt_sum[j];
           lvt_all_sum += lvtm_sum[j];
           lc_livetime[lc_index_start+j] += lvtm_sum[j];
           cmp_livetime += lvtm_sum[j];
        }
/*        printf("Res. image %3d, counts: %4d, dur: %8.2f, corr. cnt.rate data counts %4d, livetm: %6.1f s\n",
           ri_pckt, ri_evts, ri_duration, cnt_all_sum, lvt_all_sum);
*/
        if (lcbins > 32) {
           printf("Number of requested light curve bins (%2d) in one RI-image exceeds cut-array capacity\n", 
                 lcbins);
           return;
        }

/*        printf("Cut values (ri_evts: %5d, lcbins: %1d):", ri_evts, lcbins+1);
*/
        for (j=0; j<=lcbins; j++) {
           cut[j] = (int)((float)(cnt_sum[j]) / (float)(cnt_all_sum) * 1000.0);
           if (j > 0) cut[j] += cut[j-1];
           cut[lcbins] = 1000;
/*           printf("    %1d/%5d", j, cut[j]); 
*/
        }
/*        printf("\n");
*/

        lc_curT += (double)(ri_duration);
        runT += (double)(ri_duration);
        listT += (double)(ri_duration);
        T_elapse[lc_index] += (float)(ri_duration);
        Tot_elapse += (double)(ri_duration); 
        
        dataptr2 = end_ptr[3] + ri_evts_acc * tab_wdth[3];	/* analyze REST-RAW */           
        if ((dataptr2 - fitsdat) >= filelen) {printf("dataptr2 beyond end !!!\n"); goto end;}
        
        ri_evts_acc += ri_evts;
              
/* XXXXXXXXXXXXX */

        for (it2=0; it2<ri_evts; it2++) {	/* loop over events in each image */

/*           if ((ri_pckt == 12) && (it2 > 1500)) 
              printf("%4d  %02x %02x %02x %02x %02x \n", it2, (int)(*(dataptr2)), (int)(*(dataptr2+1)), 
                 (int)(*(dataptr2+2)), (int)(*(dataptr2+3)), (int)(*(dataptr2+4)));
*/           
           ri_pha = (*(dataptr2)) & 0x7;
           rawpha = ri_pha * 32;

           FRspec[1][ri_pha]++;

           x = *(dataptr2+3);
           y = *(dataptr2+4);

           lc_index = MonteCarlo(lc_index_start, lcbins, cut);
           MC_distr[lc_index-lc_index_start]++;
                   
/*           if ((ri_pckt == 12) && (it2 > 1500)) 
              printf("ri_pha: %1d, xy: %3d %3d, lc_index %4d\n", ri_pha, x, y, lc_index);
*/              
           ppha = sp_r[ri_pha] - (it2 % dsp_r[ri_pha]);	/* check this - and define sp-r and dsp_r ???????? */
              
           cpha = pha_xy(x, y, ppha, jemx, ipat, 0) * ph_corr;

           e_value = cpha * e_factor;
           
           for (ip_pha=0; ip_pha<256; ip_pha++) if (e_value < IP_tabel[ip_pha]) goto loopout3;
loopout3:  ;
           
   	   iindex = x * 256 + y;

           pha = (int)(cpha / 4.0);
           if (pha > 254) pha = 255;
                            
           pickevent = 0;
               
           iindex = 256 * (int)(x) + (int)(y);

           ee = -1;
/*           if ((pha >= pha_sel[jemx][0]) && (pha < pha_sel[jemx][1])) ee = 0;
           if ((pha >= pha_sel[jemx][1]) && (pha < pha_sel[jemx][2])) ee = 1;
           if ((pha >= pha_sel[jemx][2]) && (pha < pha_sel[jemx][3])) ee = 2;
           if ((pha >= pha_sel[jemx][3]) && (pha < pha_sel[jemx][4])) ee = 3;
*/
           if ((ri_pha >= 1) && (ri_pha < 3)) {
              ee = 0;	
              sev0++;
              shdgr[0][x][y]++;
           }
           if ((ri_pha >= 3) && (ri_pha < 5)) {
              ee = 1;
              sev1++;
              shdgr[1][x][y]++;
           }
           if (ri_pha == 5) {
              ee = 2;
              sev2++;
              shdgr[2][x][y]++;
           }
           if (ri_pha >= 6) {
              ee = 3;
              sev3++;
              shdgr[3][x][y]++;
           }

/*           if ((ri_pckt == 12) && (it2 > 1500)) 
              printf("RI ri_pha; %1d, ppha: %4d, cpha: %6.1f, pha: %3d, x/y: %3d/%3d, lc_index: %4d, ee: %1d\n", 
              ri_pha, ppha, cpha, pha, x, y, lc_index, ee);
*/              
           if (ee >= 0) {
              if (soft_shdgr[ee][iindex] > 0.0) {
                 tot_ipspec[ip_pha]++;
                 tot_spec[(int)(cpha / 4.0)]++;
              }

              if (s_pickmap[0][ee][iindex] == 1) {
                 lc_raw[ee][lc_index]++; 
              }
              
              if (b_pickmap[0][ee][iindex] == 1) {
                 lc_backgr[ee][lc_index]++;
              }

              if (list_out) {
                 if (initpat[iindex] > 0) 
                    fprintf(listfil, "%3d %14.4lf %3d %3d %4.1f\n", 
                       sfmap[iindex]+1, listT, rawpha, list_code, (float)(grey));
                 else
                    fprintf(listfil, "%3d %14.4lf %3d %3d %4.1f\n", 
                       0, listT, rawpha, list_code, (float)(grey));
              }
                 
              for (xx=1; xx<xx_max; xx++) {
                 if (s_pickmap[xx][ee][iindex] == 1) lc_rawx[xx][ee] += 1.0; 
                 if (b_pickmap[xx][ee][iindex] == 1) lc_backgrx[xx][ee] += 1.0;
              }
           }

           ipha = (int)(cpha/4.0 + 0.5);
                 
           if (s_pickmap[0][ee][iindex] == 1) {
              src_rpha[src_enum] = rawpha;
              src_ipha[src_enum] = ip_pha;
              src_x[src_enum]    = x;
              src_y[src_enum]    = y;
              src_time[src_enum] = runT;
              src_spec_p[ipha]++;
              src_ipspec_p[ip_pha]++;
              m_simple_s[ee]++; 
              if (src_enum < src_events-1) src_enum++;
           }
           if (b_pickmap[0][ee][iindex] == 1) {
              bck_rpha[src_bnum] = rawpha;
              bck_ipha[src_bnum] = ip_pha;
              bck_x[src_bnum]    = x;
              bck_y[src_bnum]    = y;
              bck_time[src_bnum] = runT;
              bckgr_spec[ipha]++;
              bckgr_ipspec[ip_pha]++;
              m_simple_b[ee]++;
              if (src_bnum < bck_events-1) src_bnum++;
           }
              
           dataptr2 += tab_wdth[3];
           if ((dataptr2 - fitsdat) >= filelen) goto end;
        }         
/* XXXXXXXXXXXXXXXXXX */

           /* ----------------- END OF DECODING OF ONE RESTRICTED IMAGE ----------------- */

/*        printf("\nEvents: %6d, distribution: ", ri_evts);
        for (i=0; i<=lcbins; i++) printf(" %1d/%6d  ", lc_index_start+i, MC_distr[i]);
        printf("   ************ goto next image\n");
*/  
        ri_evts = 0;
        ri_sum_dur = 0.0;
        goto next_image;

        /* ----------------- END OF RESTRICTED IMAGING DECODING --------------------------- */

   	/* CCCCCCCCCCCCCCCCCCCCC START OF COUNT RATE DECODING CCCCCCCCCCCCCCCCCCCCCCCCC */

rate:
   	dataptr = end_ptr[0];		/* analyze RATE-PRW - if available */
        if ((dataptr - fitsdat) >= filelen) goto end;
        dataptr2 = end_ptr[1];        	/* analyze RATE-RAW  */
        if ((dataptr2 - fitsdat) >= filelen) goto end;

        for (it1=0; it1<tab_dpth[0]; it1++) {
           fits_sh    = *dataptr * 256 + *(dataptr+1);
           SSC_1[it1]   = (int)(fits_sh & 0x7FFF) + (((int)(fits_sh & 0x8000) + 0x8000) & 0x8000);

           fits_sh    = *(dataptr+4) * 256 + *(dataptr+5);
           lobt2_1[it1] = (int)(fits_sh & 0x7FFF) + (((int)(fits_sh & 0x8000) + 0x8000) & 0x8000);

           fits_sh    = *(dataptr+6) * 256 + *(dataptr+7);
           lobt1_1[it1] = (int)(fits_sh & 0x7FFF) + (((int)(fits_sh & 0x8000) + 0x8000) & 0x8000);

           fits_sh    = *(dataptr+8) * 256 + *(dataptr+9);
           lobt0_1[it1] = (int)(fits_sh & 0x7FFF) + (((int)(fits_sh & 0x8000) + 0x8000) & 0x8000);              
              
           rt_startT = (double)(lobt2_1[it1])*65536.0 + (double)(lobt1_1[it1]) + (double)(lobt0_1[it1])/65536.0;
/*           printf("Start time of count rate packet %3d: %10.4lf\n", it1, rt_startT-lc_startT);
*/           
           fits_sh    = *(dataptr+10) * 256 + *(dataptr+11);
           rowbg = ((int)(fits_sh & 0x7FFF) + (((int)(fits_sh & 0x8000) + 0x8000) & 0x8000)) << 16;
           rowbg += *(dataptr+12) * 256 + *(dataptr+13);

           fits_sh    = *(dataptr+14) * 256 + *(dataptr+15);
           n_line  = (int)(fits_sh & 0x7FFF) + (((int)(fits_sh & 0x8000) + 0x8000) & 0x8000);

           for (j=0; j<8; j++) grey_rate[j]  = *(dataptr+16+j);

           for (j=0; j<8; j++) {
              fits_sh    = *(dataptr+24+2*j) * 256 + *(dataptr+25+2*j);
              grey_line[j]  = (int)(fits_sh & 0x7FFF) + (((int)(fits_sh & 0x8000) + 0x8000) & 0x8000);
           }

   	/* CCCCCCCCCCCCCCCCCCCCC START OF COUNT RATE PACKET DECODING CCCCCCCCCCCCCCCCCCCCCCCCC */
        		/* analyze RATE-RAW  */
           
           rat_index_off = (rt_startT - lc_startT) / 0.125;
/*           printf("Cnt_rate_pckt: %4d, rat_index_offset: %5d, no. of lines: %4d\n", it1, rat_index_off, n_line);
*/           
           for (it2=0; it2<n_line; it2++) {	/* loop over count rate bins (125 ms each) */
              
              rat_index = rat_index_off + it2;
              fits_sh = *(dataptr2+2) * 256 + *(dataptr2+3);
              cnt_ratdata[rat_index] = (int)(fits_sh & 0x7FFF) + (((int)(fits_sh & 0x8000) + 0x8000) & 0x8000);
              cnt_ratgrey[rat_index] = *(dataptr2+4);
              cnt_ratfrac[rat_index] = abs(cnt_ratfrac[rat_index]);	/* mark this channel as filled */
/*              if ((it1%10) == 0) 
                 printf("cnt_rate_pckt: %3d, cnts: %5d, grey %2d, frac; %4d\n", 
                    it1, cnt_ratdata[rat_index], cnt_ratgrey[rat_index], cnt_ratfrac[rat_index]);
*/  
              dataptr2 += tab_wdth[1];
              if ((dataptr2 - fitsdat) > filelen) {
                 printf("dataptr2-fitsdat: %12d, filelen2: %12d\n", (int)(dataptr2-fitsdat), filelen);
                 return;
              } 

           }
/* ---------------------------- END OF COUNT RATE PACKET DECODING --------------------------------- */
           dataptr += tab_wdth[0];
           if ((dataptr - fitsdat) > filelen) {
              printf("dataptr-fitsdat: %12d, filelen2: %12d\n", (int)(dataptr-fitsdat), filelen);
              return;
           } 
        }

        goto restricted;
        
/* ---------------------------- END OF COUNT RATE DATA DECODING ----------------------------------- */

end:	for (ee=0; ee<4; ee++) {
           
           norm_val[0][ee] = tilt_corr * 100.0 / (difmax[0][ee] * norm_vals[0][ee]);
           
           comp_val[1][ee] = comp_val[2][ee] = comp_val[3][ee] = comp_val[4][ee] = 0.0;
   	   for (xx=1; xx<xx_max; xx++) {
              norm_val[xx][ee] = tilt_corr * 100.0 / (difmax[xx][ee] * norm_vals[xx][ee]);
              comp_val[xx][ee] = lc_rawx[xx][ee] / spix[xx][ee] - lc_backgrx[xx][ee] / bpix[xx][ee];
              comp_val[xx][ee] *= 100.0 * norm_val[xx][ee] / cmp_livetime;
/*              printf("ee: %1d, xx: %1d, src: %10.1f, bckg: %10.1f, norm_val: %10.8f\n",
                 ee, xx, lc_rawx[xx][ee], lc_backgrx[xx][ee], norm_val[xx][ee]);
*/
           }
           
           if (ee == 3) {
              printf("Normaliz. values used in lightcurves: (norm_val):   %10.8f %10.8f %10.8f %10.8f\n",
                 	norm_val[0][0], norm_val[0][1], norm_val[0][2], norm_val[0][3]);
              printf("Compens. values used in lightcurves: (comp_val[1]): %10.8f %10.8f %10.8f %10.8f\n",
                 	comp_val[1][0], comp_val[1][1], comp_val[1][2], comp_val[1][3]);
              printf("                                     (comp_val[2]): %10.8f %10.8f %10.8f %10.8f\n",
                 	comp_val[2][0], comp_val[2][1], comp_val[2][2], comp_val[2][3]);
              printf("                                     (comp_val[3]): %10.8f %10.8f %10.8f %10.8f\n",
                 	comp_val[3][0], comp_val[3][1], comp_val[3][2], comp_val[3][3]);
              printf("                                     (comp_val[4]): %10.8f %10.8f %10.8f %10.8f\n",
                 	comp_val[4][0], comp_val[4][1], comp_val[4][2], comp_val[4][3]);
           }
           
           if (lc_timestep > Tot_elapse) {
              if (ee == 0) {
                 lc_livetime[0] += lc_livetime[1];
                 T_elapse[0] += T_elapse[1];
              }
              lc_raw[ee][0] += lc_raw[ee][1];
              lc_backgr[ee][0] += lc_backgr[ee][1];
              lc_index = 0;
              lc_time[0] = lc_startT;
           }
           
           i = (int)(ijmax / 1000);
           j = ijmax % 1000;
           if (list_out) {
              fprintf(lauxfil, "%3d %1d %4d %4d %10.6f %03d %04d %03d %3d %3d\n", 
                 list_code, ee, spix[0][ee], bpix[0][ee], norm_val[0][ee], orbit, pid, pidv, i, j);
           }

           lc_used = 0;
           for (j=0; j<=lc_index; j++) {
              lc_use[j] = 0;
              if (lc_livetime[j] < lc_timestep * 0.25) continue;
              lc_used++;
              lc_use[j] = 1;
              lc_rate[ee][j] = 
                 (((float)(lc_raw[ee][j]) / spix[0][ee] - (float)(lc_backgr[ee][j]) 
                       / bpix[0][ee])) * 100.0 * norm_val[0][ee] / lc_livetime[j];

              lc_ratex[1][ee][j] = comp_val[1][ee];
              lc_ratex[2][ee][j] = comp_val[2][ee];
              lc_ratex[3][ee][j] = comp_val[3][ee];
              lc_ratex[4][ee][j] = comp_val[4][ee];

              lc_error[ee][j] = sqrt((double)(lc_raw[ee][j]) 
                 + (double)(lc_backgr[ee][j]) * spix[0][ee] * spix[0][ee] / bpix[0][ee] / bpix[0][ee]);
              lc_error[ee][j] *= 100.0 * norm_val[0][ee] / lc_livetime[j] / spix[0][ee];


/*              printf(
       "E%1d: lc_raw: %5d, spix: %5d, lc_backgr: %5d, bpix: %5d, norm: %10.8f, livetime: %5.1f, rate: %10.5f\n",
           ee, lc_raw[ee][j], spix[0][ee], lc_backgr[ee][j], bpix[0][ee], 
           norm_val[0][ee], lc_livetime[j], lc_rate[ee][j]);
*/  
              lc_srate[ee][j] = (float)(lc_raw[ee][j]) / spix[0][ee] * 100.0;   
              lc_srate[ee][j] *= 1.0 / lc_livetime[j];
              lc_brate[ee][j] = (float)(lc_backgr[ee][j]) / bpix[0][ee] * 100.0;   
              lc_brate[ee][j] *= 1.0 / lc_livetime[j];
           }
        }

        for (j=0; j<256; j++) fprintf(gnufil, "%6d %6d\n", j, tot_spec[j]);
   	fprintf(gnufil, "\n\n");

   	for (ee=0; ee<4; ee++) {
           printf("E%1d: Pixels: raw_illum: %5d, src/backgr:, %6d/%6d, mean excess diff: %5.3f,", 
              ee, npix[0], spix[0][ee], bpix[0][ee], difmax[0][ee]);
           printf(" counts src/backgr: %6.0f/%6.0f,  excess counts: %7.2f\n", 
           m_simple_s[ee], m_simple_b[ee], (m_simple_s[ee] - m_simple_b[ee] * 
              spix[0][ee] / bpix[0][ee]) * difmax[0][ee]);
        }
        
        sprintf(sumnavn, "lc_summa.txt");
        sumfil = fopen(sumnavn, "at");
        
        fprintf(sumfil, "\n pid: %3d\n", pid);        
        fprintf(sumfil, "No of pixels: source:, %6.0f, total-source: %6.0f, signal: %6.0f, background: %6.0f\n", 
           nsimple_as, nsimple_at - nsimple_as, ncross_as, ncross_ab);
        
        fprintf(sumfil, "Source area/Background area ratio: simple: %6.3f, cross: %6.3f, combined: %6.3f\n",
           backfact2, backfact1, backfact3);

        fprintf(sumfil, 
        "Events: simple source: %6.1f, simple backgr: %6.1f, cross signal: %6.1f, cross backgr: %6.1f\n", 
           msimple_s, msimple_b, mcross_s, mcross_b);
        
        fprintf(sumfil, "Excess and S/N: simple: %6.0f %6.1f,  cross: %6.0f %6.1f,  combined: %6.0f %6.1f\n",
           	ex_sim, sn_sim, ex_smear, sn_smear, ex_com, sn_com);
        
        fclose(sumfil);
        
        for (ipat=0; ipat<patnum; ipat++) cpha = pha_xy(x, y, pha, jemx, ipat, 1);
        
        src_spec_p[0] = 0;
        bckgr_spec[0] = 0;
        src_ipspec_p[0] = 0;
        bckgr_ipspec[0] = 0;

        for (i=0; i<256; i++) {
           src_cl_ipspec[i] = (float)(src_ipspec_p[i]) - (float)(bckgr_ipspec[i]) * backfact3;         
           src_iperror[i] = 
              sqrt( (double) (src_ipspec_p[i]) + (double) (bckgr_ipspec[i])  * backfact3 * backfact3 );
        }
        
        k = 4;
        for (i=0; i<48; i+=k) {
           src_cl_spec[i] = 0.0;
           src_error[i] = 0.0;
           for (j=0; j<k; j++) {
              src_cl_spec[i] += (float)(src_spec_p[i+j]) - (float)(bckgr_spec[i+j]) * backfact3;         
              src_error[i] += (float)(src_spec_p[i+j]) + (float)(bckgr_spec[i+j]) * backfact3 * backfact3;
           }
           src_cl_spec[i] /= (float)(k);
           src_error[i] = sqrt((double)(src_error[i])) / (float)(k);
           for (j=1; j<k; j++) {
              src_cl_spec[i+j] = src_cl_spec[i];
              src_error[i+j] = src_error[i];
           }
        }
        k = 8;
        for (i=48; i<96; i+=k) {
           src_cl_spec[i] = 0.0;
           src_error[i] = 0.0;
           for (j=0; j<k; j++) {
              src_cl_spec[i] += (float)(src_spec_p[i+j]) - (float)(bckgr_spec[i+j]) * backfact3;         
              src_error[i] += (float)(src_spec_p[i+j]) + (float)(bckgr_spec[i+j]) * backfact3 * backfact3;
           }
           src_cl_spec[i] /= (float)(k);
           src_error[i] = sqrt((double)(src_error[i])) / (float)(k);
           for (j=1; j<k; j++) {
              src_cl_spec[i+j] = src_cl_spec[i];
              src_error[i+j] = src_error[i];
           }
        }
        k = 16;
        for (i=96; i<256; i+=k) {
           src_cl_spec[i] = 0.0;
           src_error[i] = 0.0;
           for (j=0; j<k; j++) {
              src_cl_spec[i] += (float)(src_spec_p[i+j]) - (float)(bckgr_spec[i+j]) * backfact3;         
              src_error[i] += (float)(src_spec_p[i+j]) + (float)(bckgr_spec[i+j]) * backfact3 * backfact3;
           }
           src_cl_spec[i] /= (float)(k);
           src_error[i] = sqrt((double)(src_error[i])) / (float)(k);
           for (j=1; j<k; j++) {
              src_cl_spec[i+j] = src_cl_spec[i];
              src_error[i+j] = src_error[i];
           }
        }

        dtst = lc_timestep / 86400.0 / 5.0;
        lc_endT = lc_time[lc_index-1];
        
        for (i=0; i<DATA_POINTS; i++) {
           lc_time[i] = lc_time[i] / 86400.0 + plot_offset; /* + 52564.1; */
        }

      if (lc_used == 0) goto nolcdata;

      sprintf(filnavn, "lc.gnu");
      gnufil = fopen(filnavn, "wt");
   
      for (j=0; j<=lc_index; j++) if (lc_use[j]) fprintf(gnufil, "%10.3lf %6d\n", lc_time[j], lc_raw[1][j]);
      fprintf(gnufil, "\n\n");
      for (j=0; j<=lc_index; j++) if (lc_use[j]) fprintf(gnufil, "%10.3lf %6d\n", lc_time[j], lc_backgr[1][j]);
      fprintf(gnufil, "\n\n");
      for (j=0; j<=lc_index; j++) if (lc_use[j]) fprintf(gnufil, "%10.3lf %10.4f\n", lc_time[j], lc_livetime[j]);
      fprintf(gnufil, "\n\n");
      for (j=0; j<=lc_index; j++) if (lc_use[j]) fprintf(gnufil, "%10.3lf %10.4f\n", lc_time[j], lc_rate[1][j]);
      fprintf(gnufil, "\n\n");
      for (j=0; j<256; j++) fprintf(gnufil, "%6.3f %6d\n", j * 30.0 / 135.0, src_spec_p[j]);
      fprintf(gnufil, "\n\n");
      for (j=0; j<256; j++) fprintf(gnufil, "%6.3f %6d\n", j * 30.0 / 135.0, bckgr_spec[j]);
      fprintf(gnufil, "\n\n");
      for (j=0; j<256; j++) fprintf(gnufil, "%6.3f %6.1f\n", j * 30.0 / 135.0, src_cl_spec[j]);
      fprintf(gnufil, "\n\n");
      for (j=0; j<256; j++) fprintf(gnufil, "%6.3f %6.1f\n", j * 30.0 / 135.0, src_error[j]);
      fprintf(gnufil, "\n\n");
      for (j=0; j<256; j++) fprintf(gnufil, "%6.3f %6.1f\n", j * 30.0 / 135.0, 0.0);
      fprintf(gnufil, "\n\n");
      fclose(gnufil);

      sprintf(filnavn, "lcb.gnu");
      if (fileclear == 1) gnufil = fopen(filnavn, "wt"); else gnufil = fopen(filnavn, "at");
      for (j=0; j<=lc_index; j++) if (lc_use[j]) 
         fprintf(gnufil, "%10.3lf %10.4f\n", lc_time[j], 0.0);
      fclose(gnufil);

      sprintf(filnavn, "lc3.gnu");
      if (fileclear == 1) gnufil = fopen(filnavn, "wt"); else gnufil = fopen(filnavn, "at");
      for (j=0; j<=lc_index; j++) if (lc_use[j]) 
         fprintf(gnufil, "%10.3lf %10.4f\n", (lc_time[j]+2.0*dtst), (float)(lc_livetime[j] * 100.0 / lc_timestep));
      if ((lc_index > 10) || (pid > pidold+1)) fprintf(gnufil, "\n");
      fclose(gnufil);

      sprintf(filnavn, "lcc.gnu");
      if (fileclear == 1) gnufil = fopen(filnavn, "wt"); else gnufil = fopen(filnavn, "at");
      for (j=0; j<=lc_index; j++) 
         if (lc_use[j]) fprintf(gnufil, "%10.3lf %10d\n", (lc_time[j]+2.0*dtst), (int)(pid/10)*10);
      if ((lc_index > 0) || (pid > pidold+1)) fprintf(gnufil, "\n");
      fclose(gnufil);

      sprintf(filnavn, "lcd.gnu");
      if (fileclear == 1) gnufil = fopen(filnavn, "wt"); else gnufil = fopen(filnavn, "at");
      for (j=0; j<=lc_index; j++) 
         if (lc_use[j]) fprintf(gnufil, "%10.3lf %10d\n", (lc_time[j]+2.0*dtst), (int)(pid%10)*10);
      if ((lc_index > 0) || (pid > pidold+1)) fprintf(gnufil, "\n");
      fclose(gnufil);

      for (ee=0; ee<4; ee++) {
         sprintf(filnavn, "lc1_%1d.gnu", ee);
         if (fileclear == 1) gnufil = fopen(filnavn, "wt"); else gnufil = fopen(filnavn, "at");
         for (j=0; j<=lc_index; j++) if (lc_use[j]) 
            fprintf(gnufil, "%10.3lf %10.1f\n", (lc_time[j]+2.0*dtst), (float)(lc_raw[ee][j]) / 400.0);
         if ((lc_index > 0) || (pid > pidold+1)) fprintf(gnufil, "\n");
         fclose(gnufil);

         sprintf(filnavn, "lc2_%1d.gnu", ee);
         if (fileclear == 1) gnufil = fopen(filnavn, "wt"); else gnufil = fopen(filnavn, "at");
         for (j=0;j<=lc_index;j++) if (lc_use[j])
            fprintf(gnufil, "%10.3lf %10.1f\n", (lc_time[j]+2.0*dtst), (float)(lc_backgr[ee][j]) / 400.0);
         if ((lc_index > 0) || (pid > pidold+1)) fprintf(gnufil, "\n");
         fclose(gnufil);

         sprintf(filnavn, "lc4_%1d.gnu", ee);
         if (fileclear == 1) gnufil = fopen(filnavn, "wt"); else gnufil = fopen(filnavn, "at");
         for (j=0; j<=lc_index; j++) if (lc_use[j]) fprintf(gnufil, "%10.3lf %10.4f %10.4f\n", 
            (double)(lc_time[j]+1.0*dtst), lc_rate[ee][j], lc_error[ee][j]);
         if ((lc_index > 0) || (pid > pidold+1)) fprintf(gnufil, "\n");
         fclose(gnufil);

         sprintf(filnavn, "lc5_%1d.gnu", ee);
         if (fileclear == 1) gnufil = fopen(filnavn, "wt"); else gnufil = fopen(filnavn, "at");
         for (j=0; j<=lc_index; j++) 
            if (lc_use[j]) fprintf(gnufil, "%10.3lf %10.4f\n", (lc_time[j]+2.0*dtst), lc_srate[ee][j]*500.0);
         if ((lc_index > 0) || (pid > pidold+1)) fprintf(gnufil, "\n");
         fclose(gnufil);

         sprintf(filnavn, "lc6_%1d.gnu", ee);
         if (fileclear == 1) gnufil = fopen(filnavn, "wt"); else gnufil = fopen(filnavn, "at");
         for (j=0; j<=lc_index; j++) 
            if (lc_use[j]) fprintf(gnufil, "%10.3lf %10.4f\n", (lc_time[j]+2.0*dtst), lc_brate[ee][j]*500.0);
         if ((lc_index > 0) || (pid > pidold+1)) fprintf(gnufil, "\n");
         fclose(gnufil);

         sprintf(filnavn, "lc7_%1d.gnu", ee);
         if (fileclear == 1) gnufil = fopen(filnavn, "wt"); else gnufil = fopen(filnavn, "at");
         for (j=0; j<=lc_index; j++) 
            if (lc_use[j]) fprintf(gnufil, "%10.3lf %10.4f\n", (lc_time[j]+2.0*dtst), lc_error[ee][j]);
         if ((lc_index > 0) || (pid > pidold+1)) fprintf(gnufil, "\n");
         fclose(gnufil);

         sprintf(filnavn, "lc8_%1d.gnu", ee);
         if (fileclear == 1) gnufil = fopen(filnavn, "wt"); else gnufil = fopen(filnavn, "at");
         for (j=0; j<=lc_index; j++) if (lc_use[j]) 
            fprintf(gnufil, "%10.3lf %10.1f\n", lc_time[j], (float)(spix[0][ee])/100.0);
         if (pid > pidold+1) fprintf(gnufil, "\n");
         fclose(gnufil);

         sprintf(filnavn, "lck_%1d.gnu", ee);
         if (fileclear == 1) gnufil = fopen(filnavn, "wt"); else gnufil = fopen(filnavn, "at");
         for (j=0; j<=lc_index; j++) if (lc_use[j]) 
            fprintf(gnufil, "%10.3lf %10.1f\n", lc_time[j], (float)(bpix[0][ee])/100.0);
         if (pid > pidold+1) fprintf(gnufil, "\n");
         fclose(gnufil);

         sprintf(filnavn, "lc9_%1d.gnu", ee);
         if (fileclear == 1) gnufil = fopen(filnavn, "wt"); else gnufil = fopen(filnavn, "at");
         for (j=0; j<=lc_index; j++) if (lc_use[j])
            fprintf(gnufil, "%10.3lf %10.4f\n", (lc_time[j]+2.0*dtst), norm_vals[0][ee] * 100.0);
         if ((lc_index > 0) || (pid > pidold+1)) fprintf(gnufil, "\n");
         fclose(gnufil);

         sprintf(filnavn, "lce_%1d.gnu", ee);
         if (fileclear == 1) gnufil = fopen(filnavn, "wt"); else gnufil = fopen(filnavn, "at");
         for (j=0; j<=lc_index; j++) if (lc_use[j]) fprintf(gnufil, "%10.3lf %10.4f %10.4f\n", 
            (lc_time[j]+2.0*dtst), lc_ratex[1][ee][j], lc_error[ee][j] * 1.41);
         if ((lc_index > 0) || (pid > pidold+1)) fprintf(gnufil, "\n");
         fclose(gnufil);

         sprintf(filnavn, "lcf_%1d.gnu", ee);
         if (fileclear == 1) gnufil = fopen(filnavn, "wt"); else gnufil = fopen(filnavn, "at");
         for (j=0; j<=lc_index; j++) if (lc_use[j]) fprintf(gnufil, "%10.3lf %10.4f %10.4f\n", 
            (lc_time[j]+2.0*dtst), lc_ratex[2][ee][j], lc_error[ee][j] * 1.41);
         if ((lc_index > 0) || (pid > pidold+1)) fprintf(gnufil, "\n");
         fclose(gnufil);

         sprintf(filnavn, "lcg_%1d.gnu", ee);
         if (fileclear == 1) gnufil = fopen(filnavn, "wt"); else gnufil = fopen(filnavn, "at");
         for (j=0; j<=lc_index; j++) if (lc_use[j]) fprintf(gnufil, "%10.3lf %10.4f %10.4f\n", 
            (lc_time[j]+2.0*dtst), lc_ratex[3][ee][j], lc_error[ee][j] * 1.41);
         if ((lc_index > 0) || (pid > pidold+1)) fprintf(gnufil, "\n");
         fclose(gnufil);

         sprintf(filnavn, "lch_%1d.gnu", ee);
         if (fileclear == 1) gnufil = fopen(filnavn, "wt"); else gnufil = fopen(filnavn, "at");
         for (j=0; j<=lc_index; j++) if (lc_use[j]) fprintf(gnufil, "%10.3lf %10.4f %10.4f\n", 
            (lc_time[j]+2.0*dtst), lc_ratex[4][ee][j], lc_error[ee][j] * 1.41);
         if ((lc_index > 0) || (pid > pidold+1)) fprintf(gnufil, "\n");
         fclose(gnufil);

         sprintf(filnavn, "lcj_%1d.gnu", ee);
         if (fileclear == 1) gnufil3 = fopen(filnavn, "wt"); else gnufil3 = fopen(filnavn, "at");

         sprintf(filnavn, "lci_%1d.gnu", ee);
         if (fileclear == 1) gnufil2 = fopen(filnavn, "wt"); else gnufil2 = fopen(filnavn, "at");

         sprintf(filnavn, "lc_out_%1d.txt", ee);
         outfil = fopen(filnavn, "at");
         
         sprintf(filnavn, "lca_%1d.gnu", ee);
         if (fileclear == 1) gnufil = fopen(filnavn, "wt"); else gnufil = fopen(filnavn, "at");

         for (j=0; j<=lc_index; j++) if (lc_use[j]) {
            tmp = (lc_ratex[1][ee][j] + lc_ratex[2][ee][j] + lc_ratex[3][ee][j] + lc_ratex[4][ee][j]) / 4.0;

            tmp_dif = (lc_ratex[1][ee][j] - tmp) * (lc_ratex[1][ee][j] - tmp);
            tmp_dif2 = (lc_ratex[2][ee][j] - tmp) * (lc_ratex[2][ee][j] - tmp);
            tmp_dif3 = (lc_ratex[3][ee][j] - tmp) * (lc_ratex[3][ee][j] - tmp);
            tmp_dif4 = (lc_ratex[4][ee][j] - tmp) * (lc_ratex[4][ee][j] - tmp);
            if (tmp_dif2 > tmp_dif) tmp_dif = tmp_dif2;
            if (tmp_dif3 > tmp_dif) tmp_dif = tmp_dif3;
            if (tmp_dif4 > tmp_dif) tmp_dif = tmp_dif4;
         
            error2 =  sqrt((double)(lc_error[ee][j]*lc_error[ee][j]) + tmp_dif);

            fprintf(gnufil, "%10.3lf %10.4f %10.4f\n", 
               (lc_time[j]+3.0*dtst), (float)(lc_rate[ee][j]-tmp), (float)(error2));

            fprintf(gnufil2, "%10.3lf %10.4f %10.4f\n", 
               (lc_time[j]+4.0*dtst), (float)((lc_rate[ee][j]-tmp) * expfact[ee]), (float)(error2 * expfact[ee]));

            fprintf(gnufil3, "%10.3lf %10.4f %10.4f\n", 
               (lc_time[j]+2.0*dtst), (float)(lc_rate[ee][j] * expfact[ee]), (float)(lc_error[ee][j] * expfact[ee]));

               /* prepare summary ascii file */
            
            fprintf(outfil, "%3d %3d %10.5lf %7.2f ", orbit, pid, lc_time[j], T_elapse[j]);
            fprintf(outfil, "%8.3f ", lc_livetime[j]);
            fprintf(outfil, "%6d ", spix[0][ee]);
            fprintf(outfil, "%5d ", lc_raw[ee][j]);
            fprintf(outfil, "%6d ", bpix[0][ee]);
            fprintf(outfil, "%5d ", lc_backgr[ee][j]);
            fprintf(outfil, "%6.4f ", norm_vals[0][ee]);
            fprintf(outfil, "%6.2f %5.2f", lc_rate[ee][j], lc_error[ee][j]);
            fprintf(outfil, "%6.2f ", tmp);
            fprintf(outfil, "%6.2f ", (float)(lc_rate[ee][j]-tmp));
            fprintf(outfil, "%6.4f ", expfact[ee]);
            fprintf(outfil, "%6.2f ", 
               (float)((lc_rate[ee][j]-tmp) * expfact[ee]));
            fprintf(outfil, "%6.2f ", lc_ratex[1][ee][j]);
            fprintf(outfil, "%6.2f ", lc_ratex[2][ee][j]);
            fprintf(outfil, "%6.2f ", lc_ratex[3][ee][j]);
            fprintf(outfil, "%6.2f ", lc_ratex[4][ee][j]);
            fprintf(outfil, "%3d\n", (int)(cmp_full / cmp_livetime * 100.0));

         }
         if ((lc_index > 0) || (pid > pidold+1)) {
            fprintf(gnufil, "\n"); 
            fprintf(gnufil2, "\n");
            fprintf(gnufil3, "\n");
         }
         fclose(gnufil);
         fclose(outfil);
         fclose(gnufil2);
         fclose(gnufil3);

      }
nolcdata:
      sprintf(filnavn, "ip_spectrum.txt");
      gnufil = fopen(filnavn, "wt");
   
      for (j=0; j<256; j++) fprintf(gnufil, "%6d %6.1f\n", j, src_cl_ipspec[j]);
      fprintf(gnufil, "\n\n");
      for (j=0; j<256; j++) fprintf(gnufil, "%6d %6.1f\n", j, src_iperror[j]);
      fprintf(gnufil, "\n\n");
      for (j=0; j<256; j++) fprintf(gnufil, "%6d %6d\n", j, tot_ipspec[j]);
      fprintf(gnufil, "\n\n");
      fclose(gnufil);
 
      printf("Number of 'source'/'background' events in list file: %7d / %7d\n", src_enum, src_bnum);
      
      if (shdg_out) {
         sprintf(fimnavn, "%03d_%04d_%03d_shd%1d.bin", orbit, pid, pidv, jemx+1);
         fim = fopen(fimnavn, "wb");
         fwrite(shdgr, 2, 4*65536, fim);
   
         aux_tot[0] = sev0;
         aux_tot[1] = sev1;
         aux_tot[2] = sev2;
         aux_tot[3] = sev3;

         aux_tot[4] = orbit * 65536.0 + pid;
       
         aux_tot[17] = 8;	/* pulseheight selection boundaries - nl_channels */
         aux_tot[18] = 24;
         aux_tot[19] = 48;
         aux_tot[20] = 80;
         aux_tot[21] = jemx;
       
         aux_tot[23] = lc_startT / 86400.0;
         aux_tot[24] = lc_endT / 86400.0;
         aux_tot[27] = cmp_livetime;
         aux_tot[28] = pidv;
           
         fwrite(aux_tot, 8, 30, fim);
         fclose(fim);
      }

      if (list_out) {
         fclose(listfil);
         fclose(lauxfil);
      }

      printf("Full-imaging spectrum: %5d %5d %5d %5d %5d %5d %5d %5d\n",
         FRspec[0][0], FRspec[0][1], FRspec[0][2], FRspec[0][3], 
         FRspec[0][4], FRspec[0][5], FRspec[0][6], FRspec[0][7]);
      printf("Rest-imaging spectrum: %5d %5d %5d %5d %5d %5d %5d %5d\n",
         FRspec[1][0], FRspec[1][1], FRspec[1][2], FRspec[1][3], 
         FRspec[1][4], FRspec[1][5], FRspec[1][6], FRspec[1][7]);
}

/* ******************************************************************************** */

float pha_xy(int x, int y, int pha, int jemx, int source, int print)
{
   FILE *xyfil, *afil, *locfil;
   char filenam[200], iline[200];
   static int data[2] = {0, 0};
   static float xycor[2][256][256], acor[2][256];
   static int evnum[2], spectrum[3][256];
   float cpha;
   int i, j, k, n;
   double m0, m1, m2, rms;
   
   if (print) goto skriv;

   if (data[jemx] == 0) {
      if (jemx == 0) sprintf(filenam, "%s/reference/fs_gain_cor.dat", sweethome);
      else  sprintf(filenam, "%s/reference/fm2_gain_cor.dat", sweethome);
      xyfil = fopen(filenam, "rt");
      if (xyfil == NULL) {
         printf("Could not open %s\n", filenam);
         exit(0);
      }
      
      for (j=0; j<256; j++) {
         for (k=0; k<42; k++) {
            i = k * 6;
            fgets(iline, 128, xyfil);
            sscanf(iline, "%f %f %f %f %f %f", &xycor[jemx][i][j], &xycor[jemx][i+1][j],
               &xycor[jemx][i+2][j], &xycor[jemx][i+3][j], &xycor[jemx][i+4][j], &xycor[jemx][i+5][j]);
         }
         fgets(iline, 128, xyfil);
         i = 252;
         sscanf(iline, "%f %f %f %f", 
            &xycor[jemx][i][j], &xycor[jemx][i+1][j], &xycor[jemx][i+2][j], &xycor[jemx][i+3][j]);
      }
      fclose(xyfil);
      
      m0 = m1 = m2 = 0.0;
      for (j=0; j<256; j++) {
         for (k=0; k<256; k++) {
            n = k*256 + j;
            if ((soft_shdgr[0][n]>0) || (soft_shdgr[1][n]>0) || (soft_shdgr[2][n]>0) || (soft_shdgr[3][n]>0)) {
               m0 += 1.0;
               m1 += xycor[jemx][k][j];
               m2 += xycor[jemx][k][j] * xycor[jemx][k][j];
            }
         }
      }
      m1 /= m0;
      m2 /= m0;
      rms = sqrt(m2 - m1 * m1);
      
/*    printf("jemx%1d detector nonuniformity: npix: %6.0lf, mean: %10.4lf, sigma: %10.4lf\n", jemx+1, m0, m1, rms);
*/   
      sprintf(filenam, "%s/reference/a_factors%1d.txt", sweethome, jemx+1);
      afil = fopen(filenam, "rt");
      if (afil == NULL) {
         printf("Could not open %s\n", filenam);
         exit(0);
      }
      
      for (j=0; j<256; j++) {
         fgets(iline, 128, afil);
         sscanf(iline, "%d %f", &k, &acor[jemx][j]);
      }
      fclose(afil);

      data[jemx] = 1;
   }
   
   if ((pha >= 0) && (pha < 1024)) {
      spectrum[0][(int)(pha/4)]++;
      evnum[jemx]++;
   }
   
   cpha = (float)(pha) / xycor[jemx][x][y];
   
   if ((cpha >= 0.0) && (cpha < 1024.0)) spectrum[1][(int)(cpha/4.0)]++;
   
   if (acor[jemx][x] > 0.0) cpha /= acor[jemx][x];
   
   if ((cpha >= 0.0) && (cpha < 1024.0)) spectrum[2][(int)(cpha/4.0)]++;
   
   return(cpha);
   
skriv:
   printf("JEM-X%1d counts: %6d\n", jemx+1, evnum[jemx]);
   if (evnum[jemx] > 0) {
      sprintf(filenam, "xx_%1d.gnu", source);
      locfil = fopen(filenam, "wt");
   
      for (j=0; j<256; j++) fprintf(locfil, "%6.3f %6d\n", j * 30.0 / 135.0, spectrum[0][j]);
      fprintf(locfil, "\n\n");
      for (j=0; j<256; j++) fprintf(locfil, "%6.3f %6d\n", j * 30.0 / 135.0, spectrum[1][j]);
      fprintf(locfil, "\n\n");
      for (j=0; j<256; j++) fprintf(locfil, "%6.3f %6d\n", j * 30.0 / 135.0, spectrum[2][j]);
      fprintf(locfil, "\n\n");
      fclose(locfil);
   }
   
   return(1.0);
}

/* ******************************************************************************** */

float correct(int spec[])
{
   int i, max=0, i1, i2, imax, m0, m1, backgr;
   float mean, corr;
   
   for (i=110; i<170; i++) {
      if (spec[i] > max) {
         max = spec[i];
         imax = i;
      }
   }
   
   i1 = i2 = 0;
   max = (float)(max) * 0.8;
   for (i=imax; i<256; i++) if (spec[i] > max) i2 = i;
   for (i=imax; i>0; i--) 
      if (spec[i] < max) {
         i1 = i+1;
         goto videre;
      }

videre:   
   m1 = 0;
   for (i=i1-1; i>i1-11; i--) m1 += spec[i];
   for (i=i2+1; i<i2+11; i++) m1 += spec[i];
   
/*   max = m1 / 20; */
   m1 = m0 = 0;
   for (i=i1; i<i2; i++) {
      m0 += spec[i] - max;
      m1 += (spec[i] - max) * (i - i1);
   }

   mean = (float)(i1) + (float)(m1) / (float)(m0);
   
   corr = 140.0 / mean;
   
   printf("imax: %3d, i1: %3d, i2: %3d, m1: %10d, m0: %6d, corr: %6.4f\n",
      imax, i1, i2, m0, m1, corr);
   
   return(corr);
}

/* ******************************************************************************** */

int genmap(char initpat[detdim], unsigned char sfmap[4][detdim], 
   float norm_vals[4], float coll_area[4], float bkg_aff[4], float src_ill[4], 
   char s_pickmap[4][detdim], char b_pickmap[4][detdim], int spix[4], int bpix[4], float difmax[4])
{
   int i, j, k, nee, mee, ee, N[256], imax, jmax, ijmax;
   double f[256], M0[256], M1[256], Q0[256], Q1[256], topav, botav, SN, SNmax, SNmax_old;

   for (i=0; i<256; i++) f[i] = (double)(i) / 250.0;

   for (ee=0; ee<4; ee++) {
      spix[ee] = bpix[ee] = 0;
      for (i=0; i<256; i++) {
         M0[i] = M1[i] = Q0[i] = Q1[i] = 0.0;
         N[i] = 0;
      } 
      for (i=0; i<256; i++) {
         for (j=0; j<detdim; j++) {
            if (((int)(sfmap[ee][j]) == i) && (soft_shdgr[ee][j] > 0.0) && (initpat[j] > 0)) N[i]++; 
         }
      }
/*      for (i=0; i<256; i++) {
         if ((i%16) == 0) printf("\n");
         printf(" %4d", N[i]);
      }
      printf("\n");
*/      
      for (j=0; j<detdim; j++) {
         if (initpat[j] > 1) src_ill[ee] += 1.0;		
            		/* integrate total source illuminated area in pixels */
         if ((initpat[j] > 0) &&  (soft_shdgr[ee][j] > 0.0)) bkg_aff[ee] += sfmap[ee][j] / 250.0;
            		/* integrate total illuminated and useful area in pixels */
         s_pickmap[ee][j] = b_pickmap[ee][j] = 0.0;
      }
      
      Q0[0] = N[0];
      M0[250] = N[250];
      Q1[0] = 0.0;
      M1[250] = N[250] * f[250];
      for (i=1; i<250; i++) {
         Q0[i] = Q0[i-1] + N[i];
         M0[250-i] = M0[250-i+1] + N[250-i];
         Q1[i] = Q1[i-1] + N[i] * f[i];
         M1[250-i] = M1[250-i+1] + N[250-i] * f[250-i];
/*         printf("%3d  M0: %10.4lf, M1: %10.4lf,   %3d  Q0: %10.4lf, Q1: %10.4lf,  N: %10d\n", 
            250-i, M0[250-i], M1[250-i], i, Q0[i], Q1[i], N[i]);
*/
      }
      
      SNmax = 0.0;
      for (i=250; i>5; i--) {
         if (N[i] == 0) continue;
         topav = M1[i] / M0[i];
         for (j=0; j<i; j++) {
            if (N[j] == 0) continue;
            botav = Q1[j] / Q0[j];
            SN = (topav - botav) / sqrt(1.0 / M0[i] + 1.0 / Q0[j]);
            if (SN > SNmax) {
               SNmax = SN;
               imax = i;
               jmax = j;
               difmax[ee] = topav-botav;
            }
         }
      }
/*
printf("ee: %1d, i: %3d, M0: %6.0lf, j: %3d, Q0: %6.0lf, difmax: %10.6lf nominal SNmax: %10.6lf, n_v: %10.4f\n", 
               ee, imax, M0[imax], jmax, Q0[jmax], difmax[ee], SNmax, norm_vals[ee]);
*/  
      coll_area[ee] = M0[imax] * difmax[ee] * norm_vals[ee];
      
      for (j=0; j<detdim; j++) {
         if ((sfmap[ee][j] > imax) && (soft_shdgr[ee][j] > 0.0)) {
            s_pickmap[ee][j] = 1.0; 
            spix[ee]++;
         }
         if ((sfmap[ee][j] < jmax) && (initpat[j] > 0) && (soft_shdgr[ee][j] > 0.0)) {
            b_pickmap[ee][j] = 1.0; 
            bpix[ee]++;
         }
      }
      
   }
   printf("Total source illuminated area in pixels: [src_ill]     %8.1f   %8.1f   %8.1f   %8.1f\n",
      src_ill[0], src_ill[1], src_ill[2], src_ill[3]);
   printf("Background affected area in pixels: [bkg_aff]          %8.1f   %8.1f   %8.1f   %8.1f\n",
      bkg_aff[0], bkg_aff[1], bkg_aff[2], bkg_aff[3]);
   printf("Extracted pixel area (in lightcurves): [coll_area]     %8.1f   %8.1f   %8.1f   %8.1f\n",
      coll_area[0], coll_area[1], coll_area[2], coll_area[3]);
   printf("Source extraction efficiency: [difmax]                 %8.4f   %8.4f   %8.4f   %8.4f\n",
      difmax[0], difmax[1], difmax[2], difmax[3]);
   printf("No. of source pixels used in lightcurves: [spix]       %8d   %8d   %8d   %8d\n",
      spix[0], spix[1], spix[2], spix[3]);
   printf("No. of backgr. pixels used in lightcurves: [bpix]      %8d   %8d   %8d   %8d\n",
      bpix[0], bpix[1], bpix[2], bpix[3]);
   printf("Normalization values used to form sfmap: [norm_vals]  %10.8f %10.8f %10.8f %10.8f\n",
      norm_vals[0], norm_vals[1], norm_vals[2], norm_vals[3]);

   ijmax = imax * 1000 + jmax;
   return(ijmax);
}

/* ******************************************************************************** */

int src_map(int jemx, float src_RA, float src_dec, int orbit, int pid, char detmap[detdim], 
                unsigned char src_maps[4][detdim], float norm_vals[4], float src_xy[2])
{
      double p_RA[10000], p_dec[10000], p_posang[10000];
      float srcx, srcy, norm_d, track_list[100], comb_list[500], absolen, scatt_list[500];
      float sum, sum2, lysfrac[4], srcmap[detdim];
      double sep, sx, sy, sz, px, py, pz;
      int track_len, track_delta[100], comb_len, comb_delta[500], scatt_len, scatt_delta[500];
      int pix_cnt[4], max_pix, i, n, j, nn, sxy, last = 0, ee;

/*      printf("Calling getRAdec_init\n");
*/
      i = getRAdec_init(jemx, orbit, pid, p_RA, p_dec, p_posang);
/*      printf("Return from getRAdec_init, i=%6d\n", i);
*/
      if (i == 0) return(-1);		/* no pointing data found */

/*      printf("src_map:  source RA/dec: %10.6f %10.6f\n", src_RA, src_dec);
*/

      px = cos(p_dec[pid] * gtorad) * cos(p_RA[pid] * gtorad);
      py = cos(p_dec[pid] * gtorad) * sin(p_RA[pid] * gtorad);
      pz = sin(p_dec[pid] * gtorad);

      getxy(src_RA, src_dec, jemx, &srcx, &srcy);

      sx = cos((double)(src_dec) * gtorad) * cos((double)(src_RA) * gtorad);
      sy = cos((double)(src_dec) * gtorad) * sin((double)(src_RA) * gtorad);
      sz = sin((double)(src_dec) * gtorad);
      
      sep = sqrt((px-sx)*(px-sx) + (py-sy)*(py-sy) + (pz-sz)*(pz-sz));
      if (sep > 0.10) return(-1);	/* source not visible */

      sxy = (int)(srcx) * sky_ydim + (int)(srcy);
      src_xy[0] = srcx;
      src_xy[1] = srcy;
      
      printf("\nFirst function call (sxy=%5d, jmx=%1d) with new source position: %6.2f, %6.2f, last %1d\n", 
   		sxy, jemx+1, srcx, srcy, last);

      norm_d = detecmap(srcx, srcy, detmap, last, jemx);	

      if (norm_d < 0.0) {
         printf("No data for sky_index %05d\n", sxy);
         return(0);
      }  

      for (ee=0; ee<4; ee++) {

         /* calculate detector illumination from (fractional) source position */
         /* detmap contains 2 in illuminated detector pixels, and 1 in area onto which mask pattern is projected */

         for (i=1; i<detdim; i++) {
            srcmap[i] = 0.0;
            accupat[i] = 0.0;
         }
         
         absolen = abs_len[ee];
         track_len =  sh_depth(absolen, sxy, track_list, track_delta);
         /* calculate distribution of X-ray absorbtion points relative to entry point at window level */
         /* track_list contains fraction of (1) photon arriving at detector pixels specified in track_delta */ 

/*         sum = 0.0;
         for (i=0; i<track_len; i++) sum += track_list[i];
         printf("Sum of %3d contributions along track1 %7.4f at energy %1d\n", track_len, sum, ee);
*/
            /* calculate the position scatter around the X-ray absorbtion point at this energy */
         
         scatt_len = sh_xy(x_rms[ee], y_rms[ee], scatt_list, scatt_delta); 

/*         sum = 0.0;
         for (i=0; i<scatt_len; i++) sum += scatt_list[i];
         printf("Sum of %3d contributions around each track point1 %7.4f at energy %1d\n", scatt_len, sum, ee);
*/
         for (i=0; i<detdim; i++) accupat[i] = 0.0;
         
         /* calculate combined effect of X-ray penetration into detector and position uncertainty of MS read-out */
         comb_len = shapecomb2(scatt_list, scatt_delta, scatt_len, 
    			track_list, track_delta, track_len, comb_list, comb_delta, accupat, 1.0);
         /* comb_list contains fraction of 1 photon arriving at detector pixels specified in comb_delta */            

/*         sum = 0.0;
         for (i=0; i<comb_len; i++) sum += comb_list[i];
         printf("Sum of %3d contributions around complete track1 %7.4f at energy %1d\n", comb_len, sum, ee);
*/
         /* repeat calculation for upper energy bound of this range */
         
         absolen = abs_len[ee+1];
         track_len =  sh_depth(absolen, sxy, track_list, track_delta);
         /* calculate distribution of X-ray absorbtion points relative to entry point at window level */
         /* track_list contains fraction of (1) photon arriving at detector pixels specified in track_delta */ 

/*         sum = 0.0;
         for (i=0; i<track_len; i++) sum += track_list[i];
         printf("Sum of %3d contributions along track2: %7.4f at energy %1d\n", track_len, sum, ee);
*/
            /* calculate the position scatter around the X-ray absorbtion point at this energy */
         
         scatt_len = sh_xy(x_rms[ee+1], y_rms[ee+1], scatt_list, scatt_delta); 

/*         sum = 0.0;
         for (i=0; i<scatt_len; i++) sum += scatt_list[i];
         printf("Sum of %3d contributions around each track point2: %7.4f at energy %1d\n", scatt_len, sum, ee);
*/         
         /* calculate combined effect of X-ray penetration into detector and position uncertainty of MS read-out */
         comb_len = shapecomb2(scatt_list, scatt_delta, scatt_len, 
    			track_list, track_delta, track_len, comb_list, comb_delta, accupat, acc_w[ee]);
         /* comb_list contains fraction of 1 photons arriving at detector pixels specified in comb_delta */            
         
/*         sum = 0.0;
         for (i=0; i<comb_len; i++) sum += comb_list[i];
         printf("Sum of %3d contributions around complete track1+2: %7.4f (weights: %3d/%3d) at energy %1d\n", 
            comb_len, sum, (int)(acc_w[ee] * 100.0), (int)((1.0-acc_w[ee]) * 100.0), ee);
*/    
         pix_cnt[ee] = shapefold(detmap, comb_list, comb_delta, comb_len, srcmap, last);
         /* srcmap contains for each detector pixel the calculated fraction of 1 photon arriving from the source */
      
         sum2 = 0.0;
         lysfrac[ee] = 0.0;
         for (i=1; i<detdim; i++) {
            if (soft_shdgr[ee][i] <= 0.0) srcmap[i] = 0.0;
            if (srcmap[i] > lysfrac[ee]) {
               lysfrac[ee] = srcmap[i];
            }
         }
      
         nn = 0;
         for (i=1; i<detdim; i++) {
            sfmap[ee][i] = (unsigned char)(srcmap[i] * 250.0 / lysfrac[ee]);
               /* rescale the pixel illumination to the range 0-250 */
            if (detmap[i] > 1) nn++;	/* count number of illuminated pixels (at the window) */
         }
         norm_vals[ee] = lysfrac[ee];
      }
/*      printf("Return from src_map, lysfrac: %10.5f %10.5f %10.5f %10.5f\n", 
         lysfrac[0], lysfrac[1], lysfrac[2], lysfrac[3]);
*/      
      return(nn);
}

/* ************************************************************************************* */
int fitshead(unsigned char* datptr, unsigned char* end_ptr[], int tab_wdth[], 
   int tab_dpth[], char E_name[], int extnum, int filelen, double st_T[])
{

   unsigned char *extp, *nax1, *nax2, *xnam, save, *e_ptr, xtension[30], temp[50], *namp; 
   int i, tab_w, tab_d, xnum, status, lobt2, lobt1, lobt0;
   short fits_sh;
   double startT;

   sprintf(xtension, "XTENSION= 'BINTABLE'");
   sprintf(E_name, "NULL");
   
nextext:
   if ((datptr - fitsdat) == filelen) {
      status = 0; 
      return(status);
   }
   if ((datptr - fitsdat) > filelen) {
      status = -1; 
      return(status);
   }
   
   extp = strstr(datptr, xtension);
   if (extp == NULL) {
      datptr += 2880;
      goto nextext;
   }
   
   if ((e_ptr = strstr(extp, "END    ")) == NULL) {printf("null-pointer 03\n"); status=-11; return(status);}

   if ((e_ptr - fitsdat) % 80 != 0) {
      printf("END -marker found out of place\n");
      status = -12; 
      return(status);
   }
           
   if (((e_ptr - fitsdat) % 2880) > 0) e_ptr = fitsdat + ((int)((e_ptr - fitsdat) / 2880) + 1) * 2880;
        
   xnum = 9;
   if ((namp = strstr(extp, "EXTNAME")) == NULL) {status = -1; return(status);}
   if ((int)(e_ptr - namp) < 0) {status = 0; return(status);}
   strncpy(E_name, namp+16, 8);
   E_name[8] = 0;

   if (strcmp(E_name, "RATE-PRW") == 0) xnum = 0;
   if (strcmp(E_name, "RATE-RAW") == 0) xnum = 1;
   if (strcmp(E_name, "REST-PRW") == 0) xnum = 2;
   if (strcmp(E_name, "REST-RAW") == 0) xnum = 3;
   if (strcmp(E_name, "REST-SRW") == 0) xnum = 4;
   if (strcmp(E_name, "FULL-PRW") == 0) xnum = 5;
   if (strcmp(E_name, "FULL-RAW") == 0) xnum = 6;
   if (strcmp(E_name, "FULL-SRW") == 0) xnum = 7;
   if (strcmp(E_name, "REST-PHA") == 0) xnum = 8;

   if ((nax1 = strstr(extp, "NAXIS1")) == NULL)  {status = -3; return(status);}
   if ((int)(e_ptr - nax1) < 0) {status = -4; return(status);}

   strncpy(temp, nax1+12, 32);
   tab_w = atoi(temp);
           
   if ((nax2 = strstr(extp, "NAXIS2")) == NULL) {status = -5; return(status);}
   if ((int)(e_ptr - nax2) < 0) {status = -6; return(status);}

   strncpy(temp, nax2+12, 32);
   tab_d = atoi(temp);
          
   if ((xnum == 0) || (xnum == 2) || (xnum == 5)) {
      fits_sh    = *(e_ptr+4) * 256 + *(e_ptr+5);
      lobt2 = (int)(fits_sh & 0x7FFF) + (((int)(fits_sh & 0x8000) + 0x8000) & 0x8000);

      fits_sh    = *(e_ptr+6) * 256 + *(e_ptr+7);
      lobt1 = (int)(fits_sh & 0x7FFF) + (((int)(fits_sh & 0x8000) + 0x8000) & 0x8000);

      fits_sh    = *(e_ptr+8) * 256 + *(e_ptr+9);
      lobt0 = (int)(fits_sh & 0x7FFF) + (((int)(fits_sh & 0x8000) + 0x8000) & 0x8000);              
             
      startT = (double)(lobt2) * 65536.0 + (double)(lobt1);
              /* ensure that lc_startT falls on a count rate channel boundary 
                 (just in case we get restricted imaging data in this science window) */
      startT += (double)((unsigned)(lobt0) % 0x0000F600) / 65536.0;	
   } else startT = 0.0;
   
   if (xnum < 9) {
      end_ptr[xnum] = e_ptr;
      tab_wdth[xnum] = tab_w;
      tab_dpth[xnum] = tab_d;
      st_T[xnum] = startT;
      printf("E_name: %s, depth: %10d, width: %4d, data start: %12d, length: %12d\n", 
         E_name, tab_d, tab_w, (int)(e_ptr-fitsdat), (int)(tab_w*tab_d));
   }
   
   datptr = e_ptr + tab_d*tab_w;
   if (((datptr - fitsdat) % 2880) != 0)   
      datptr = fitsdat + ((int)((datptr - fitsdat) / 2880) + 1) * 2880;
/*   printf("Goto nextext with datpoint: %12d\n", (int)(datptr - fitsdat));
*/
   goto nextext;
}


/* *************************************************************************** */

int MonteCarlo(int lc_index_start, int lcbins, int cut[])
{
   int lc_index, i, j;
 
   i = (rand() % 1000);
   for (j=0; j<=lcbins; j++) {
      if (i < cut[j]) return(lc_index_start + j);
   }
   return(lc_index_start + lcbins);
}
