#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/timeb.h>

#include "midi_b.h"

double shapefold(char det[], float comb_list[], int comb_delta[], int comb_len, float det_out[], int last);
int sh_xy(double x_rms, double y_rms, float scatt_list[], int scatt_delta[]);

/* *********************************************************** */

int sh_xy(double x_rms, double y_rms, float scatt_list[], int scatt_delta[])
{
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                             */        
/*    Calculate the distribution of actual detection points    *
/*    relative to the photon interaction position on a 1x1 mm2 */
/*    grid. The rms uncertainty in the X and Y position        */
/*    determination (in mm) are given in x_rms and y_rms.      */             
/*                                                             */
/*    Niels Lund, February 2003                                */        
/*                                                             */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */

/*    Calculate the distribution of actual detection points relative to the
      photon interaction position on a 1x1 mm2 grid. The rms uncertainty in
      the X and Y position determination (in mm) are given in x_rms and y_rms.
*/      
      int i, j, k, m, n, nold, delta_i[41][41];
      double x_dist[21], y_dist[21], id, t, f, pi_c, q, sum, x, y, xfac, yfac, x_gauss[21], y_gauss[21];

      pi_c = 1.0 / sqrt(2.0 * 3.14159);
      
      for (i=0; i<21; i++) {
         id = (double)(2 * i + 1);
         x = 0.5 * id / x_rms;
         f = pi_c * exp(-x * x / 2.0);
         x_gauss[i] = f;
         t = 1.0 / (1.0 + 0.2316419 * x);
         q = f * t * (0.31938153 - t * (.356563782 - t * (1.781477937 - t * (1.821255978 - t * 1.330274429))));
         /* Abramowitz & Stegun, Hdbk of Math. Funct., NBS 1968 */
         x_dist[i] = q; 

         y = 0.5 * id / y_rms;
         f = pi_c * exp(-y * y / 2.0);
         y_gauss[i] = f;
         t = 1.0 / (1.0 + 0.2316419 * y);
         q = f * t * (0.31938153 - t * (.356563782 - t * (1.781477937 - t * (1.821255978 - t * 1.330274429))));
         /* Abramowitz & Stegun, Hdbk of Math. Funct., NBS 1968 */
         y_dist[i] = q; 
      }

      x_dist[20] = x_dist[19];
      y_dist[20] = y_dist[19];
      for (i=19; i>0; i--) {
         x_dist[i] = x_dist[i-1] - x_dist[i];
         y_dist[i] = y_dist[i-1] - y_dist[i];
      }
      x_dist[0] = 1.0 - 2.0 * x_dist[0];
      y_dist[0] = 1.0 - 2.0 * y_dist[0];
      
      sum = -x_dist[0];
      for (i=0; i<21; i++) {
         id = (double)(2 * i + 1);
         x = 0.5 * id / x_rms;
         sum += 2 * x_dist[i];
      }
      sum = -y_dist[0];
      for (i=0; i<21; i++) {
         id = (double)(2 * i + 1);
         y = 0.5 * id / y_rms;
         sum += 2 * y_dist[i];
      }
      n = 0;
      for (i=-20; i<=20; i++) {
         k = i + 20;
         xfac = x_dist[(int)(abs(i))];         
         for (j=-20; j<=20; j++) {
            m = j + 20;
            yfac = y_dist[(int)(abs(j))];
            nold = n;
            if (xfac * yfac > 0.0001) {
               scatt_list[n] = xfac * yfac;
               scatt_delta[n] = i * det_ydim + j;
               n++;
               if (n >= 500) {printf("Scatt_list length exceeded\n"); return(-1);}
            }
         }
      }

      return(n);
}

/* *********************************************************** */

double shapefold(char det[], float comb_list[], int comb_delta[], int comb_len, float det_out[], int last)
{
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                             */        
/*  shapefold.c calculates the distribution of one photon      */
/*  arriving at "window" pixel (X,Y) among the neighboring     */        
/*  pixels at the microstrip plate.                            */
/*                                                             */
/*  Niels Lund, February 2003                                  */        
/*                                                             */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */

   int i, j, k;
   double sum1, loss1;

   for (i=0; i<detdim; i++) det_out[i] = 0.0;
   
   sum1 = loss1 = 0.0;
   for (i=1; i<detdim; i++) {
      if (det[i] > 1) {
         sum1 += 1.0;
         for (j=0; j<comb_len; j++) {
            k = i + comb_delta[j];
            if ((k < 0) || (k >= detdim)) 
               loss1 += comb_list[j];
            else
               det_out[k] += comb_list[j];
         }
      }
   }
   
   if (last) printf("final sum1: %5.3lf  loss1: %5.3lf\n", sum1, loss1);
   return(sum1);
}

/* *********************************************************** */

int sh_depth(double abs_len, int sky_index, float track_list[], int track_delta[])
{
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                             */        
/*  sh_depth calculates the penetration of X-rays down into    */
/*  the detector gas. sh_depth calculates which section of     */
/*  a given track which projects to a certain mm interval on   */
/*  the microstrip plate. Which sections are relevant at a     */
/*  given energy is calculated at the end of the routine.      */        
/*                                                             */
/*  Niels Lund, February 2003                                  */        
/*                                                             */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */

/*	Calculate track lengths projecting to individual detector pixels for photons penetrating 
   	into the detector.
   	The photon is assumed to come from a sky-direction "sky_index" and to enter the detector 
        at detector pixel (0,0) at the window level.
        For calculation purposes each 1x1 mm2 detector pixel is subdivided into 4x4 subpixels.
        The first two array indices in the arrays "det_step", "track" and "kt_max" are the sub_pixel coords.
        The last index in "det_step" and "track" indicates the number of sequential wall crossings 
        for the track going from pixel to pixel (1x1 mm2 pixels) as it penetrates down the depth of the
        detector. The last entry in these two arrays corresponds to the point where the track exits the
        bottom of the detector.
        The array "det_step" contains the 1x1 mm2 detector pixel coordinates for the traversed pixels.
           (actually the one-dmensional coordinate difference with respect to the entry point)
        The array "track" contains the corresponding track length (in mm) measured from the entry point
        at the window until the track leaves this detector pixel (entering another pixel or leaving 
        through the detector bottom).
        kt_max is a control array holding the largest index value used for the third index of
        det_step and track (must be less than 16!)
*/
        int i, j, ix, iy, kx, ky, kt, sky_x, sky_y, ibot_x[4], ibot_y[4], max_kx[8], max_ky[8];
        int k, m, n, kt_max[4][4], det_step[4][4][16], det_xy0, track_len, det_index, w;
        float track_l_x[8][4], track_l_y[8][4], t, alfa_x, alfa_y, proj_max_x, proj_max_y, wx, wy;
        float top_x[4], top_y[4], wallx_z[8][4], wally_z[8][4], full_track, bot_x[4], bot_y[4];
        float track[4][4][16], escape, x_rms, y_rms, sum, det_depth = -55.0;
        float det[detdim];
        
           /* check that the mask_height is well defined (external parameter) */
        if ((mask_height < 3390.0) || (mask_height > 3403.0)) {
           printf("In m_shapes: mask_height value outside acceptable range: %6.2lf\n", mask_height);
           exit(0);
        }
        det_xy0 = 128 * 256 + 128;
        
        for (k=0; k<4; k++) {
           for (m=0; m<4; m++) {
              kt_max[k][m] = 0;
              for (n=0; n<16; n++) {
                 track[k][m][n] = 0.0;
                 det_step[k][m][n] = 0;
              }
           }
        }

        for (i=0; i<8; i++) {
           max_kx[i] = max_ky[i] = 0;
        }
        
        if (sky_index >= skydim) return(-1);
   	sky_x = sky_index / sky_ydim;
        sky_y = sky_index % sky_ydim;
        sky_y -= (sky_ydim - 1) / 2;	/* sky-coordinates in a "sky_ydim" (=255) scale */
        sky_x -= (sky_xdim - 1) / 2;
        sky_x *= 3.0;	/* sky-coordinates in a "3*sky_ydim" (=765) scale */
        sky_y *= 3.0;	/* this scale is convenient for the calculations on the sub-pixel scale */

        alfa_x = (double)(sky_x) / mask_height;
        alfa_y = (double)(sky_y) / mask_height;
        
        proj_max_x = alfa_x * det_depth;
        proj_max_y = alfa_y * det_depth;


/*        printf("Sh_depth: abs_len: %6.2lf, sky_index: %6d, sky_x/y: %+4d/%+4d, proj_max_x/y: %+5.1f/%+5.1f\n",
           abs_len, sky_index, sky_x, sky_y, proj_max_x, proj_max_y);
*/        
        /* calculate depth (Z-coordinate) of intersection of sky line with X-coordinate boundaries */
        /* max depth (negative!) is -55 mm */
        ix = 0;
        
        for (i=-3; i<=3; i+=2) {			/* loop over sub-pixel collumns */
           kx = 0;
           wx = 0.5;
           top_x[ix] = (double)(i) / 8.0;
           bot_x[ix] = top_x[ix] + proj_max_x;
           ibot_x[ix] = (int)(bot_x[ix] + 256.5) - 256;	/* itop_x is by definition allways 0 */

           if (sky_x == 0) {
              wallx_z[kx][0] = wallx_z[kx][1] = wallx_z[kx][2] = wallx_z[kx][3] = det_depth; 
              max_kx[0] = max_kx[1] = max_kx[2] = max_kx[3] = 1;
              goto noshift_x;
           }
more_kx:           
           if (sky_x > 0) wallx_z[kx][ix] = -(wx + top_x[ix]) * mask_height / sky_x;
           else wallx_z[kx][ix] = (wx - top_x[ix]) * mask_height / sky_x;
           max_kx[ix] = kx + 1;
           if (max_kx[ix] > 8) return(-2);
        
           if (wallx_z[kx][ix] > det_depth) {kx++; wx += 1.0; goto more_kx;}      
           else wallx_z[kx][ix] = det_depth;  
           
           ix++;
        }
noshift_x:

        /* calculate depth (Z-coordinate) of intersection of sky line with Y-coordinate boundaries */
        /* max depth (negative!) is -55 mm */
        wy = 0.5;
        iy = 0;
        for (i=-3; i<=3; i+=2) {			/* loop over sub-pixel rows */
           ky = 0;
           wy = 0.5;
           top_y[iy] = (double)(i) / 8.0;
           bot_y[iy] = top_y[iy] + proj_max_y;
           ibot_y[iy] = (int)(bot_y[iy] + 256.5) - 256;	/* itop_y is by definition allways 0 */

           if (sky_y == 0) {
              wally_z[ky][0] = wally_z[ky][1] = wally_z[ky][2] = wally_z[ky][3] = det_depth; 
              max_ky[0] = max_ky[1] = max_ky[2] = max_ky[3] = 1;
              goto noshift_y;
           }
more_ky:           
           if (sky_y > 0) wally_z[ky][iy] = -(wy + top_y[iy]) * mask_height / sky_y;
           else wally_z[ky][iy] = (wy - top_y[iy]) * mask_height / sky_y;
           max_ky[iy] = ky + 1;
           if (max_ky[iy] > 8) return(-3);
        
           if (wally_z[ky][iy] > det_depth) {ky++; wy += 1.0; goto more_ky;}      
           else wally_z[ky][iy] = det_depth;  
           
           iy++;
        }
noshift_y:

/*   	printf("Wall crossings in X: 0: %1d,  1: %1d,  2: %1d,  3: %1d\n",
           max_kx[0], max_kx[1], max_kx[2], max_kx[3]);

   	printf("Wall crossings in Y: 0: %1d,  1: %1d,  2: %1d,  3: %1d\n",
           max_ky[0], max_ky[1], max_ky[2], max_ky[3]);
*/
   	/* calculate track length and (X,Y)-coordinates for each wall intersection */
        
        full_track = sqrt((det_depth*det_depth) + 
           		  (top_x[0]-bot_x[0])*(top_x[0]-bot_x[0]) +
           		  (top_y[0]-bot_y[0])*(top_y[0]-bot_y[0]));
        

        for (ix=0; ix<4; ix++) {			/* loop over sub-pixel collums */
           for (kx=0; kx<max_kx[ix]; kx++) {		/* loop over pixel walls in x */
              track_l_x[kx][ix] = full_track * wallx_z[kx][ix] / det_depth;
           }
        }
           
        for (iy=0; iy<4; iy++) {			/* loop over sub-pixel rows */
           for (ky=0; ky<max_ky[iy]; ky++) {		/* loop over pixel walls in y */
              track_l_y[ky][iy] = full_track * wally_z[ky][iy] / det_depth;
           }
        }           
        
        /* sort the wall crossings sequentially according to the track length */
        
        w = 0;
        for (i=0; i<4; i++) {			/* loop over sub-pixel X rows */
           for (j=0; j<4; j++) {		/* loop over sub-pixel Y collumns */
              ky = kx = kt = 0;
igen:         if ((kx == max_kx[i]) && (ky == max_ky[j])) goto nextj;         
   	      if (kx == max_kx[i]) goto stepy;
              if (ky == max_ky[j]) goto stepx;
   	      if (track_l_x[kx][i] <= track_l_y[ky][j]) {	/* next crossing in X */
stepx:           t = track[i][j][kt] = track_l_x[kx][i];		/* track length from window to this X-wall */
                 det_step[i][j][kt] = -kx * sign(sky_x) * det_ydim - ky * sign(sky_y); /* corresp. sky_index step */
                 if (fabs(t - track_l_y[ky][j]) < 0.005) ky++; 
                    /* may cross X and Y walls (or detector bottom) simultaneously */
                 kx++;                 
              } else {			                 	/* next crossing in Y */
stepy:           t = track[i][j][kt] = track_l_y[ky][j];		/* track length from window to this Y-wall */
                 det_step[i][j][kt] = -kx * sign(sky_x) * det_ydim - ky * sign(sky_y); /* corresp. sky_index step */
                 ky++;
              }
              
              kt++;
              w++;
              if (kt > 16) return(-4);
              
              kt_max[i][j] = kt;

              if (t < full_track) goto igen;
nextj:	      ;
           }
        }
        
/*        printf("Total number of wall crossings: %2d\n", w);
*/        

        for (i=0; i<detdim; i++) det[i] = 0.0;
        
        for (k=0; k<4; k++) {
           for (m=0; m<4; m++) {
              for (n=0; n<kt_max[k][m]; n++) {
                 escape = exp(-track[k][m][n] / abs_len);
                 if (n == 0) det[det_xy0 + det_step[k][m][n]] += 1.0 - escape; 
                 else det[det_xy0 + det_step[k][m][n]] += exp(-track[k][m][n-1] / abs_len) - escape; 
              }
           }
        }
        
        sum = 0.0;
        for (k=7; k>=-7; k--) {
           for (m=-7; m<=7; m++) {
              det_index = det_xy0 + m * 256 + k;
/*              printf(" %5d", (int)(det[det_index] * 1000.0 / 16.0 + 0.5));
*/
              sum += det[det_index] / 16.0;
           }
/*           printf("\n");
*/
        }


        track_len = 0;
        for (i=0; i<detdim; i++) {
           if (det[i] > 0.0) {
              track_list[track_len] = det[i] / 16.0;
              track_delta[track_len] = det_xy0 - i;	/* sign inverted 4/2 2003 nl */
              track_len++;
              if (track_len > 100) {printf("More than 100 track entries\n"); return(-5);}
           }
        }

/*        printf("Track entries: %3d\n", track_len);
*/        
/*        printf("sh_depth contribution sum: %6.4lf  (o/oo)\n", sum);
*/
        return(track_len);
}

/* *********************************************************** */

int shapecomb(float scatt_list[], int scatt_delta[], int scatt_len, 
   	float track_list[], int track_delta[], int track_len, float comb_list[], int comb_delta[])
        
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++  */
/*                                                              */        
/*  shapecomb.c combines the output from a "smearing"           */
/*  routine: sh_depth.c [track_list/_delta] and a penetration   */
/*  sobroutines: sh_xy.c [scatt_list/_delta] and forms the      */
/*  combined smearing effect in [comb_list/_delta] to be        */
/*  applied to the idealized shadowgram generated by detecmap.c */
/*                                                              */
/*  Niels Lund, February 2003                                   */        
/*                                                              */
/* ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */

{
   int i, j, k, n, i0, i1;
   float temp[detdim];

   for (i=0; i<detdim; i++) temp[i] = 0.0;
   
   i0 = 64 * det_ydim + 128;
   
   for (j=0; j<track_len; j++) {
      k = i0 + track_delta[j];
      temp[k] += track_list[j];
   }
   
   for (i=0; i<32768; i++) {
      if (temp[i] > 0.0) {
         for (j=0; j<scatt_len; j++) {         
            k = 32768 + i + scatt_delta[j];
            temp[k] += temp[i] * scatt_list[j];
         }
      }
   }

   n = 0;
   i1 = i0 + 32768;
   for (i=32768; i<detdim; i++) 
       if (temp[i] > 0.0) {
          comb_list[n] = temp[i];
          comb_delta[n] = i - i1;
          n++;
          if (n > 500) return(-1);
       }
   
   return(n);
}

/* *********************************************************** */

int sign(int x)
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
/*                                                             */        
/*  sign.c returns +1 for positive integers and -1 for         */
/*  negative integers.                                         */        
/*                                                             */
/*  Niels Lund, February 2003                                  */        
/*                                                             */
/* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ */
{
   if (x) return(x/abs(x));
   return(1);
}

